@@ -18,27 +18,21 @@
 #endif
 
 #include <string>
+#include <vector>
 #include <map>
 
-#include "Platform.h"
-
-#if PLAT_GTK
+#if defined(__unix__)
 
 #include <unistd.h>
+
+#if defined(GTK)
 #include <gtk/gtk.h>
-
 #endif
 
-#if PLAT_WIN
+#else
 
-#ifdef __BORLANDC__
-// Borland includes Windows.h for STL and defaults to different API number
-#ifdef _WIN32_WINNT
 #undef _WIN32_WINNT
-#endif
-#endif
-
-#define _WIN32_WINNT  0x0400
+#define _WIN32_WINNT  0x0500
 #ifdef _MSC_VER
 // windows.h, et al, use a lot of nameless struct/unions - can't fix it, so allow it
 #pragma warning(disable: 4201)
@@ -54,22 +48,21 @@
 #ifdef _MSC_VER
 #include <direct.h>
 #endif
-#ifdef __BORLANDC__
-#include <dir.h>
-#endif
 
 #endif
 
-#include "SciTE.h"
-#include "PropSet.h"
+#include "Scintilla.h"
+
+#include "GUI.h"
+
 #include "SString.h"
 #include "StringList.h"
-#include "Accessor.h"
-#include "WindowAccessor.h"
-#include "Scintilla.h"
-#include "Extender.h"
+#include "StringHelpers.h"
 #include "FilePath.h"
 #include "PropSetFile.h"
+#include "StyleWriter.h"
+#include "Extender.h"
+#include "SciTE.h"
 #include "Mutex.h"
 #include "JobQueue.h"
 #include "SciTEBase.h"
@@ -147,20 +140,20 @@ void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
 	if (end < 0)
 		end = lengthDoc;
 	RemoveFindMarks();
-	SendEditor(SCI_COLOURISE, 0, -1);
+	wEditor.Call(SCI_COLOURISE, 0, -1);
 
 	// Read the default settings
 	char key[200];
 	sprintf(key, "style.*.%0d", STYLE_DEFAULT);
-	char *valdef = StringDup(props.GetExpanded(key).c_str());
+	char *valdefDefault = StringDup(props.GetExpanded(key).c_str());
 	sprintf(key, "style.%s.%0d", language.c_str(), STYLE_DEFAULT);
-	char *val = StringDup(props.GetExpanded(key).c_str());
+	char *valDefault = StringDup(props.GetExpanded(key).c_str());
 
-	StyleDefinition defaultStyle(valdef);
-	defaultStyle.ParseStyleDefinition(val);
+	StyleDefinition defaultStyle(valdefDefault);
+	defaultStyle.ParseStyleDefinition(valDefault);
 
-	delete []val;
-	delete []valdef;
+	delete []valDefault;
+	delete []valdefDefault;
 
 	int tabSize = props.GetInt("export.rtf.tabsize", props.GetInt("tabsize"));
 	int wysiwyg = props.GetInt("export.rtf.wysiwyg", 1);
@@ -183,7 +176,7 @@ void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
 	if (tabSize == 0)
 		tabSize = 4;
 
-	FILE *fp = saveName.Open("wt");
+	FILE *fp = saveName.Open(GUI_TEXT("wt"));
 	if (fp) {
 		char styles[STYLE_DEFAULT + 1][MAX_STYLEDEF];
 		char fonts[STYLE_DEFAULT + 1][MAX_FONTDEF];
@@ -278,7 +271,7 @@ void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
 		        RTF_BOLD_OFF RTF_ITALIC_OFF, defaultStyle.size);
 		bool prevCR = false;
 		int styleCurrent = -1;
-		WindowAccessor acc(wEditor.GetID(), props);
+		TextReader acc(wEditor);
 		int column = 0;
 		for (i = start; i < end; i++) {
 			char ch = acc[i];
@@ -323,7 +316,7 @@ void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
 		fputs(RTF_BODYCLOSE, fp);
 		fclose(fp);
 	} else {
-		SString msg = LocaliseMessage("Could not save file '^0'.", filePath.AsFileSystem());
+		GUI::gui_string msg = LocaliseMessage("Could not save file '^0'.", filePath.AsInternal());
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 	}
 }
@@ -333,7 +326,7 @@ void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
 
 void SciTEBase::SaveToHTML(FilePath saveName) {
 	RemoveFindMarks();
-	SendEditor(SCI_COLOURISE, 0, -1);
+	wEditor.Call(SCI_COLOURISE, 0, -1);
 	int tabSize = props.GetInt("tabsize");
 	if (tabSize == 0)
 		tabSize = 4;
@@ -344,7 +337,7 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 	int titleFullPath = props.GetInt("export.html.title.fullpath", 0);
 
 	int lengthDoc = LengthDocument();
-	WindowAccessor acc(wEditor.GetID(), props);
+	TextReader acc(wEditor);
 
 	bool styleIsUsed[STYLE_MAX + 1];
 	if (onlyStylesUsed) {
@@ -363,17 +356,17 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 	}
 	styleIsUsed[STYLE_DEFAULT] = true;
 
-	FILE *fp = saveName.Open("wt");
+	FILE *fp = saveName.Open(GUI_TEXT("wt"));
 	if (fp) {
 		fputs("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n", fp);
 		fputs("<html xmlns=\"http://www.w3.org/1999/xhtml\">\n", fp);
 		fputs("<head>\n", fp);
 		if (titleFullPath)
 			fprintf(fp, "<title>%s</title>\n",
-			        static_cast<const char *>(filePath.AsFileSystem()));
+			        static_cast<const char *>(filePath.AsUTF8().c_str()));
 		else
 			fprintf(fp, "<title>%s</title>\n",
-			        static_cast<const char *>(filePath.Name().AsFileSystem()));
+			        static_cast<const char *>(filePath.Name().AsUTF8().c_str()));
 		// Probably not used by robots, but making a little advertisement for those looking
 		// at the source code doesn't hurt...
 		fputs("<meta name=\"Generator\" content=\"SciTE - www.Scintilla.org\" />\n", fp);
@@ -424,6 +417,9 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 		delete []val;
 		delete []valdef;
 
+		SString sval = props.GetExpanded("font.monospace");
+		StyleDefinition sdmono(sval.c_str());
+
 		for (int istyle = 0; istyle <= STYLE_MAX; istyle++) {
 			if ((istyle > STYLE_DEFAULT) && (istyle <= STYLE_LASTPREDEFINED))
 				continue;
@@ -436,6 +432,13 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 				StyleDefinition sd(valdef);
 				sd.ParseStyleDefinition(val);
 
+				if (CurrentBuffer()->useMonoFont && sd.font.length() && sdmono.font.length()) {
+					sd.font = sdmono.font;
+					sd.size = sdmono.size;
+					sd.italics = sdmono.italics;
+					sd.bold = sdmono.bold;
+				}
+
 				if (sd.specified != StyleDefinition::sdNone) {
 					if (istyle == STYLE_DEFAULT) {
 						fprintf(fp, "span {\n");
@@ -660,8 +663,8 @@ void SciTEBase::SaveToHTML(FilePath saveName) {
 		fputs("\n</body>\n</html>\n", fp);
 		fclose(fp);
 	} else {
-		SString msg = LocaliseMessage(
-		            "Could not save file \"^0\".", filePath.AsFileSystem());
+		GUI::gui_string msg = LocaliseMessage(
+		            "Could not save file \"^0\".", filePath.AsInternal());
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 	}
 }
@@ -814,7 +817,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 		int fontSize;		// properties supplied by user
 		int fontSet;
 		int pageWidth, pageHeight;
-		PRectangle pageMargin;
+		GUI::Rectangle pageMargin;
 		//
 		PDFRender() {
 			pageStarted = false;
@@ -1026,7 +1029,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 	PDFRender pr;
 
 	RemoveFindMarks();
-	SendEditor(SCI_COLOURISE, 0, -1);
+	wEditor.Call(SCI_COLOURISE, 0, -1);
 	// read exporter flags
 	int tabSize = props.GetInt("tabsize", PDF_TAB_DEFAULT);
 	if (tabSize < 0) {
@@ -1053,7 +1056,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 	if (0 >= (pr.pageWidth = atol(buffer))) {
 		pr.pageWidth = PDF_WIDTH_DEFAULT;
 	}
-	next = GetNextPropItem(next, buffer, 32);
+	GetNextPropItem(next, buffer, 32);
 	if (0 >= (pr.pageHeight = atol(buffer))) {
 		pr.pageHeight = PDF_HEIGHT_DEFAULT;
 	}
@@ -1121,10 +1124,10 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 	}
 	delete []buffer;
 
-	FILE *fp = saveName.Open("wb");
+	FILE *fp = saveName.Open(GUI_TEXT("wb"));
 	if (!fp) {
 		// couldn't open the file for saving, issue an error message
-		SString msg = LocaliseMessage("Could not save file '^0'.", filePath.AsFileSystem());
+		GUI::gui_string msg = LocaliseMessage("Could not save file '^0'.", filePath.AsInternal());
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 		return;
 	}
@@ -1135,7 +1138,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 
 	// do here all the writing
 	int lengthDoc = LengthDocument();
-	WindowAccessor acc(wEditor.GetID(), props);
+	TextReader acc(wEditor);
 
 	if (!lengthDoc) {	// enable zero length docs
 		pr.nextLine();
@@ -1228,14 +1231,14 @@ static void defineTexStyle(StyleDefinition &style, FILE* fp, int istyle) {
 
 void SciTEBase::SaveToTEX(FilePath saveName) {
 	RemoveFindMarks();
-	SendEditor(SCI_COLOURISE, 0, -1);
+	wEditor.Call(SCI_COLOURISE, 0, -1);
 	int tabSize = props.GetInt("tabsize");
 	if (tabSize == 0)
 		tabSize = 4;
 
 	char key[200];
 	int lengthDoc = LengthDocument();
-	WindowAccessor acc(wEditor.GetID(), props);
+	TextReader acc(wEditor);
 	bool styleIsUsed[STYLE_MAX + 1];
 
 	int titleFullPath = props.GetInt("export.tex.title.fullpath", 0);
@@ -1249,7 +1252,7 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 	}
 	styleIsUsed[STYLE_DEFAULT] = true;
 
-	FILE *fp = saveName.Open("wt");
+	FILE *fp = saveName.Open(GUI_TEXT("wt"));
 	if (fp) {
 		fputs("\\documentclass[a4paper]{article}\n"
 		      "\\usepackage[a4paper,margin=2cm]{geometry}\n"
@@ -1277,7 +1280,7 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 
 		fputs("\\begin{document}\n\n", fp);
 		fprintf(fp, "Source File: %s\n\n\\noindent\n\\small{\n",
-		        static_cast<const char *>(titleFullPath ? filePath.AsFileSystem() : filePath.Name().AsFileSystem()));
+		        static_cast<const char *>(titleFullPath ? filePath.AsUTF8().c_str() : filePath.Name().AsUTF8().c_str()));
 
 		int styleCurrent = acc.StyleAt(0);
 
@@ -1343,8 +1346,8 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 		fputs("}\n} %end small\n\n\\end{document}\n", fp); //close last empty style macros and document too
 		fclose(fp);
 	} else {
-		SString msg = LocaliseMessage(
-		            "Could not save file \"^0\".", filePath.AsFileSystem());
+		GUI::gui_string msg = LocaliseMessage(
+		            "Could not save file \"^0\".", filePath.AsInternal());
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 	}
 }
@@ -1383,7 +1386,7 @@ void SciTEBase::SaveToXML(FilePath saveName) {
 	// but will eventually use utf-8 (once i know how to get them out).
 
 	RemoveFindMarks();
-	SendEditor(SCI_COLOURISE, 0, -1);
+	wEditor.Call(SCI_COLOURISE, 0, -1);
 
 	int tabSize = props.GetInt("tabsize");
 	if (tabSize == 0) {
@@ -1392,20 +1395,20 @@ void SciTEBase::SaveToXML(FilePath saveName) {
 
 	int lengthDoc = LengthDocument();
 
-	WindowAccessor acc(wEditor.GetID(), props);
+	TextReader acc(wEditor);
 
-	FILE *fp = saveName.Open("wt");
+	FILE *fp = saveName.Open(GUI_TEXT("wt"));
 
 	if (fp) {
 
 		bool collapseSpaces = (props.GetInt("export.xml.collapse.spaces", 1) == 1);
 		bool collapseLines  = (props.GetInt("export.xml.collapse.lines", 1) == 1);
 
-		fputs("<?xml version='1.0' encoding='ascii'?>\n", fp);
+		fprintf(fp, "<?xml version='1.0' encoding='%s'?>\n", (codePage == SC_CP_UTF8) ? "utf-8" : "ascii");
 
 		fputs("<document xmlns='http://www.scintila.org/scite.rng'", fp);
 		fprintf(fp, " filename='%s'",
-		        static_cast<const char *>(filePath.Name().AsFileSystem()));
+		        static_cast<const char *>(filePath.Name().AsUTF8().c_str()));
 		fprintf(fp, " type='%s'", "unknown");
 		fprintf(fp, " version='%s'", "1.0");
 		fputs(">\n", fp);
@@ -1523,7 +1526,7 @@ void SciTEBase::SaveToXML(FilePath saveName) {
 
 		fclose(fp);
 	} else {
-		SString msg = LocaliseMessage("Could not save file \"^0\".", filePath.AsFileSystem());
+		GUI::gui_string msg = LocaliseMessage("Could not save file \"^0\".", filePath.AsInternal());
 		WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
 	}
 }
