@@ -13,23 +13,27 @@
 #include <sys/stat.h>
 #include <time.h>
 
-#include "Platform.h"
+#include <string>
+#include <vector>
+#include <algorithm>
 
-#if PLAT_GTK
+#if defined(__unix__)
 
 #include <unistd.h>
+
+#if defined(GTK)
 #include <gtk/gtk.h>
+#endif
 
 #include <dirent.h>
 #include <errno.h>
 
-#endif
-
-#if PLAT_WIN
+#else
 
 #include <io.h>
 
-#define _WIN32_WINNT  0x0400
+#undef _WIN32_WINNT
+#define _WIN32_WINNT  0x0500
 #ifdef _MSC_VER
 // windows.h, et al, use a lot of nameless struct/unions - can't fix it, so allow it
 #pragma warning(disable: 4201)
@@ -47,9 +51,6 @@
 #ifdef _MSC_VER
 #include <direct.h>
 #endif
-#ifdef __BORLANDC__
-#include <dir.h>
-#endif
 #ifdef __DMC__
 #include <dir.h>
 #endif
@@ -59,50 +60,43 @@
 
 #endif
 
+#include "Scintilla.h"
+
+#include "GUI.h"
+
 #include "SString.h"
-#include "PropSet.h"
 #include "FilePath.h"
 
-#ifdef unix
-const char pathSepString[] = "/";
-const char pathSepChar = '/';
-const char listSepString[] = ":";
-const char configFileVisibilityString[] = ".";
-const char fileRead[] = "rb";
-const char fileWrite[] = "wb";
-#endif
-#ifdef __vms
-const char pathSepString[] = "/";
-const char pathSepChar = '/';
-const char listSepString[] = ":";
-const char configFileVisibilityString[] = "";
-const char fileRead[] = "r";
-const char fileWrite[] = "w";
+#if defined(__unix__)
+const GUI::gui_char pathSepString[] = "/";
+const GUI::gui_char pathSepChar = '/';
+const GUI::gui_char listSepString[] = ":";
+const GUI::gui_char configFileVisibilityString[] = ".";
+const GUI::gui_char fileRead[] = "rb";
+const GUI::gui_char fileWrite[] = "wb";
 #endif
 #ifdef WIN32
 // Windows
-const char pathSepString[] = "\\";
-const char pathSepChar = '\\';
-const char listSepString[] = ";";
-const char configFileVisibilityString[] = "";
-const char fileRead[] = "rb";
-const char fileWrite[] = "wb";
+const GUI::gui_char pathSepString[] = GUI_TEXT("\\");
+const GUI::gui_char pathSepChar = '\\';
+const GUI::gui_char listSepString[] = GUI_TEXT(";");
+const GUI::gui_char configFileVisibilityString[] = GUI_TEXT("");
+const GUI::gui_char fileRead[] = GUI_TEXT("rb");
+const GUI::gui_char fileWrite[] = GUI_TEXT("wb");
 #endif
 
-FilePath::FilePath(const char *fileName_) : fileName(fileName_) {}
+FilePath::FilePath(const GUI::gui_char *fileName_) : fileName(fileName_ ? fileName_ : GUI_TEXT("")) {}
+
+FilePath::FilePath(const GUI::gui_string &fileName_) : fileName(fileName_) {}
 
 FilePath::FilePath(FilePath const &directory, FilePath const &name) {
 	Set(directory, name);
 }
 
-void FilePath::Set(const char *fileName_) {
+void FilePath::Set(const GUI::gui_char *fileName_) {
 	fileName = fileName_;
 }
 
-const char *FilePath::AsFileSystem() const {
-	return AsInternal();
-}
-
 void FilePath::Set(FilePath const &other) {
 	fileName = other.fileName;
 }
@@ -112,8 +106,9 @@ void FilePath::Set(FilePath const &directory, FilePath const &name) {
 		fileName = name.fileName;
 	} else {
 		fileName = directory.fileName;
-		fileName.appendwithseparator(name.fileName.c_str(),
-			fileName.endswith(pathSepString) ? '\0' : pathSepChar);
+		if (fileName.size() && (fileName[fileName.size()-1] != pathSepChar))
+			fileName += pathSepString;
+		fileName += name.fileName;
 	}
 }
 
@@ -123,12 +118,13 @@ void FilePath::SetDirectory(FilePath directory) {
 }
 
 void FilePath::Init() {
-	fileName = "";
+	fileName = GUI_TEXT("");
 }
 
-bool FilePath::SameNameAs(const char *other) const {
+bool FilePath::SameNameAs(const GUI::gui_char *other) const {
 #ifdef WIN32
-	return EqualCaseInsensitive(fileName.c_str(), other);
+	return CSTR_EQUAL == CompareString(LOCALE_SYSTEM_DEFAULT, NORM_IGNORECASE,
+		fileName.c_str(), -1, other, -1);
 #else
 	return fileName == other;
 #endif
@@ -143,18 +139,14 @@ bool FilePath::IsSet() const {
 }
 
 bool FilePath::IsUntitled() const {
-	const char *dirEnd = strrchr(AsInternal(), pathSepChar);
-	return !dirEnd || !dirEnd[1];
+	size_t dirEnd = fileName.rfind(pathSepChar);
+	return (dirEnd == GUI::gui_string::npos) || (!fileName[dirEnd+1]);
 }
 
 bool FilePath::IsAbsolute() const {
 	if (fileName.length() == 0)
 		return false;
-#ifdef unix
-	if (fileName[0] == '/')
-		return true;
-#endif
-#ifdef __vms
+#ifdef __unix__
 	if (fileName[0] == '/')
 		return true;
 #endif
@@ -168,7 +160,7 @@ bool FilePath::IsAbsolute() const {
 
 bool FilePath::IsRoot() const {
 #ifdef WIN32
-    if ((fileName[0] == pathSepChar) && (fileName[1] == pathSepChar) && (fileName.search(pathSepString, 2) < 0))
+	if ((fileName[0] == pathSepChar) && (fileName[1] == pathSepChar) && (fileName.find(pathSepString, 2) == GUI::gui_string::npos))
         return true; // UNC path like \\server
 	return (fileName.length() == 3) && (fileName[1] == ':') && (fileName[2] == pathSepChar);
 #else
@@ -184,92 +176,152 @@ int FilePath::RootLength() {
 #endif
 }
 
-const char *FilePath::AsInternal() const {
+const GUI::gui_char *FilePath::AsInternal() const {
 	return fileName.c_str();
 }
 
+std::string FilePath::AsUTF8() const {
+	return GUI::UTF8FromString(fileName);
+}
+
 FilePath FilePath::Name() const {
-	const char *dirEnd = strrchr(fileName.c_str(), pathSepChar);
-	if (dirEnd)
-		return dirEnd + 1;
+	size_t dirEnd = fileName.rfind(pathSepChar);
+	if (dirEnd != GUI::gui_string::npos)
+		return fileName.substr(dirEnd + 1);
 	else
-		return fileName.c_str();
+		return fileName;
 }
 
 FilePath FilePath::BaseName() const {
-	const char *dirEnd = strrchr(fileName.c_str(), pathSepChar);
-	const char *extStart = strrchr(fileName.c_str(), '.');
-	if (dirEnd) {
+	size_t dirEnd = fileName.rfind(pathSepChar);
+	size_t extStart = fileName.rfind('.');
+	if (dirEnd != GUI::gui_string::npos) {
 		if (extStart > dirEnd) {
-			return FilePath(SString(dirEnd + 1, 0, extStart - dirEnd - 1).c_str());
+			return FilePath(fileName.substr(dirEnd + 1, extStart - dirEnd - 1));
 		} else {
-			return FilePath(dirEnd + 1);
+			return FilePath(fileName.substr(dirEnd + 1));
 		}
 	} else if (extStart) {
-		return FilePath(SString(fileName.c_str(), 0, extStart - fileName.c_str()).c_str());
+		return FilePath(fileName.substr(0, extStart));
 	} else {
-		return fileName.c_str();
+		return fileName;
 	}
 }
 
 FilePath FilePath::Extension() const {
-	const char *dirEnd = strrchr(fileName.c_str(), pathSepChar);
-	const char *extStart = strrchr(fileName.c_str(), '.');
-	if (extStart > dirEnd)
-		return extStart + 1;
+	size_t dirEnd = fileName.rfind(pathSepChar);
+	size_t extStart = fileName.rfind('.');
+	if ((extStart != GUI::gui_string::npos) && (extStart > dirEnd))
+		return fileName.substr(extStart + 1);
 	else
-		return "";
+		return GUI_TEXT("");
 }
 
 FilePath FilePath::Directory() const {
 	if (IsRoot()) {
 		return FilePath(fileName.c_str());
 	} else {
-		const char *dirEnd = strrchr(fileName.c_str(), pathSepChar);
-		if (dirEnd) {
-			int lenDirectory = dirEnd - fileName.c_str();
-			if (lenDirectory < RootLength()) {
-				lenDirectory = RootLength();
+		size_t lenDirectory = fileName.rfind(pathSepChar);
+		if (lenDirectory != GUI::gui_string::npos) {
+			if (lenDirectory < static_cast<size_t>(RootLength())) {
+				lenDirectory = static_cast<size_t>(RootLength());
 			}
-			return FilePath(fileName.substr(0, lenDirectory).c_str());
+			return FilePath(fileName.substr(0, lenDirectory));
 		} else {
 			return FilePath();
 		}
 	}
 }
 
-static char *split(char*& s, char c) {
-	char *t = s;
+#ifdef _WIN32
+
+// Substitute functions that take wchar_t arguments but have the same name
+// as char functions so that the compiler will choose the right form.
+
+static size_t strlen(const wchar_t *str) {
+	return wcslen(str);
+}
+
+static wchar_t *strcpy(wchar_t *strDestination, const wchar_t *strSource) {
+	return wcscpy(strDestination, strSource);
+}
+
+static int strcmp(const wchar_t *a, const wchar_t *b) {
+	return wcscmp(a,b);
+}
+
+static wchar_t *strchr(wchar_t *str, wchar_t c) {
+	return wcschr(str, c);
+}
+
+static wchar_t *strrchr(wchar_t *str, wchar_t c) {
+	return wcsrchr(str, c);
+}
+
+static wchar_t *getcwd(wchar_t *buffer, int maxlen) {
+	return _wgetcwd(buffer, maxlen);
+}
+
+static int chdir(const wchar_t *dirname) {
+	return _wchdir(dirname);
+}
+
+static FILE *fopen(const wchar_t *filename, const wchar_t *mode) {
+	return _wfopen(filename, mode);
+}
+
+static int unlink(const wchar_t *filename) {
+	return _wunlink(filename);
+}
+
+static int access(const wchar_t *path, int mode) {
+	return _waccess(path, mode);
+}
+
+#if defined(_MSC_VER) && (_MSC_VER > 1310)
+static int stat(const wchar_t *path, struct _stat64i32 *buffer) {
+	return _wstat(path, buffer);
+}
+#else
+static int stat(const wchar_t *path, struct _stat *buffer) {
+	return _wstat(path, buffer);
+}
+#endif
+
+#endif
+
+static GUI::gui_char *split(GUI::gui_char*& s, GUI::gui_char c) {
+	GUI::gui_char *t = s;
 	if (s && (s = strchr(s, c)) != NULL)
 		* s++ = '\0';
 	return t;
 }
 
 FilePath FilePath::NormalizePath() const {
-	char *path = new char[fileName.length() + 1];
+	GUI::gui_char *path = new GUI::gui_char[fileName.length() + 1];
 	strcpy(path, AsInternal());
 #ifdef WIN32
 	// Convert unix path separators to Windows
-	for (char *cp = path; *cp; cp++) {
+	for (GUI::gui_char *cp = path; *cp; cp++) {
 		if (*cp == '/')
 			*cp = pathSepChar;
 	}
 #endif
-	char *absPath = new char[fileName.length() + 1];
-	char *cur = absPath;
+	GUI::gui_char *absPath = new GUI::gui_char[fileName.length() + 1];
+	GUI::gui_char *cur = absPath;
 	*cur = '\0';
-	char *tmp = path;
+	GUI::gui_char *tmp = path;
 	if (*tmp == pathSepChar) {
 		*cur++ = pathSepChar;
 		*cur = '\0';
 		tmp++;
 	}
-	char *part;
+	GUI::gui_char *part;
 	while ((part = split(tmp, pathSepChar)) != NULL) {
-		char *last;
-		if (strcmp(part, ".") == 0)
+		GUI::gui_char *last;
+		if (strcmp(part, GUI_TEXT(".")) == 0)
 			;
-		else if (strcmp(part, "..") == 0 && (last = strrchr(absPath, pathSepChar)) != NULL) {
+		else if (strcmp(part, GUI_TEXT("..")) == 0 && (last = strrchr(absPath, pathSepChar)) != NULL) {
 			if (last > absPath)
 				cur = last;
 			else
@@ -288,66 +340,6 @@ FilePath FilePath::NormalizePath() const {
 	return ret;
 }
 
-#ifdef __vms
-
-FilePath FilePath::VMSToUnixStyle() {
-	// possible formats:
-	// o disk:[dir.dir]file.type
-	// o logical:file.type
-	// o [dir.dir]file.type
-	// o file.type
-	// o /disk//dir/dir/file.type
-	// o /disk/dir/dir/file.type
-
-	char unixStyleFileName[MAX_PATH + 20];
-	const char *vmsName = FullPath();
-
-	if (strchr(vmsName, ':') == NULL && strchr(vmsName, '[') == NULL) {
-		// o file.type
-		// o /disk//dir/dir/file.type
-		// o /disk/dir/dir/file.type
-		if (strstr (vmsName, "//") == NULL) {
-			return FilePath(vmsName);
-		}
-		strcpy(unixStyleFileName, vmsName);
-		char *p;
-		while ((p = strstr (unixStyleFileName, "//")) != NULL) {
-			strcpy (p, p + 1);
-		}
-		return FilePath(unixStyleFileName);
-	}
-
-	// o disk:[dir.dir]file.type
-	// o logical:file.type
-	// o [dir.dir]file.type
-
-	if (vmsName [0] == '/') {
-		strcpy(unixStyleFileName, vmsName);
-	} else {
-		unixStyleFileName [0] = '/';
-		strcpy(unixStyleFileName + 1, vmsName);
-		char *p = strstr(unixStyleFileName, ":[");
-		if (p == NULL) {
-			// o logical:file.type
-			p = strchr(unixStyleFileName, ':');
-			*p = '/';
-		} else {
-			*p = '/';
-			strcpy(p + 1, p + 2);
-			char *end = strchr(unixStyleFileName, ']');
-			if (*end != NULL) {
-				*end = '/';
-			}
-			while (p = strchr(unixStyleFileName, '.'), p != NULL && p < end) {
-				*p = '/';
-			}
-		}
-	}
-	return FilePath(unixStyleFileName);
-} // VMSToUnixStyle
-
-#endif
-
 /**
  * Take a filename or relative path and put it at the end of the current path.
  * If the path is absolute, return the same path.
@@ -356,10 +348,10 @@ FilePath FilePath::AbsolutePath() const {
 #ifdef WIN32
 	// The runtime libraries for GCC and Visual C++ give different results for _fullpath
 	// so use the OS.
-	char absPath[2000];
+	GUI::gui_char absPath[2000];
 	absPath[0] = '\0';
-	LPTSTR fileBit = 0;
-	::GetFullPathNameA(AsFileSystem(), sizeof(absPath), absPath, &fileBit);
+	GUI::gui_char *fileBit = 0;
+	::GetFullPathNameW(AsInternal(), sizeof(absPath)/sizeof(absPath[0]), absPath, &fileBit);
 	return FilePath(absPath);
 #else
 	if (IsAbsolute()) {
@@ -373,7 +365,7 @@ FilePath FilePath::AbsolutePath() const {
 // Only used on Windows to fix the case of file names
 
 FilePath FilePath::GetWorkingDirectory() {
-	char dir[MAX_PATH + 1];
+	GUI::gui_char dir[MAX_PATH + 1];
 	dir[0] = '\0';
 	if (getcwd(dir, MAX_PATH)) {
 		dir[MAX_PATH] = '\0';
@@ -388,27 +380,25 @@ FilePath FilePath::GetWorkingDirectory() {
 }
 
 bool FilePath::SetWorkingDirectory() const {
-	return chdir(AsFileSystem()) == 0;
+	return chdir(AsInternal()) == 0;
 }
 
-void FilePath::FixCase() {}
-
 void FilePath::List(FilePathSet &directories, FilePathSet &files) {
 #ifdef WIN32
-	FilePath wildCard(*this, "*.*");
+	FilePath wildCard(*this, GUI_TEXT("*.*"));
 	bool complete = false;
-	WIN32_FIND_DATA findFileData;
-	HANDLE hFind = ::FindFirstFile(wildCard.AsFileSystem(), &findFileData);
+	WIN32_FIND_DATAW findFileData;
+	HANDLE hFind = ::FindFirstFileW(wildCard.AsInternal(), &findFileData);
 	if (hFind != INVALID_HANDLE_VALUE) {
 		while (!complete) {
-			if ((strcmp(findFileData.cFileName, ".") != 0) && (strcmp(findFileData.cFileName, "..") != 0)) {
+			if ((strcmp(findFileData.cFileName, GUI_TEXT(".")) != 0) && (strcmp(findFileData.cFileName, GUI_TEXT("..")) != 0)) {
 				if (findFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
 					directories.Append(FilePath(AsInternal(), findFileData.cFileName));
 				} else {
 					files.Append(FilePath(AsInternal(), findFileData.cFileName));
 				}
 			}
-			if (!::FindNextFile(hFind, &findFileData)) {
+			if (!::FindNextFileW(hFind, &findFileData)) {
 				complete = true;
 			}
 		}
@@ -433,13 +423,11 @@ void FilePath::List(FilePathSet &directories, FilePathSet &files) {
 		}
 	}
 
-	if (errno == 0) {
-		closedir(dp);
-	}
+	closedir(dp);
 #endif
 }
 
-FILE *FilePath::Open(const char *mode) const {
+FILE *FilePath::Open(const GUI::gui_char *mode) const {
 	if (IsSet()) {
 		return fopen(fileName.c_str(), mode);
 	} else {
@@ -448,21 +436,29 @@ FILE *FilePath::Open(const char *mode) const {
 }
 
 void FilePath::Remove() const {
-	unlink(AsFileSystem());
+	unlink(AsInternal());
 }
 
 #ifndef R_OK
-// Neither Borland nor Microsoft define the constants used to call access
+// Microsoft does not define the constants used to call access
 #define R_OK 4
 #endif
 
 time_t FilePath::ModifiedTime() const {
 	if (IsUntitled())
 		return 0;
-	if (access(AsFileSystem(), R_OK) == -1)
+	if (access(AsInternal(), R_OK) == -1)
 		return 0;
+#ifdef _WIN32
+#if defined(_MSC_VER) && (_MSC_VER > 1310)
+	struct _stat64i32 statusFile;
+#else
+	struct _stat statusFile;
+#endif
+#else
 	struct stat statusFile;
-	if (stat(AsFileSystem(), &statusFile) != -1)
+#endif
+	if (stat(AsInternal(), &statusFile) != -1)
 		return statusFile.st_mtime;
 	else
 		return 0;
@@ -495,8 +491,16 @@ bool FilePath::Exists() const {
 }
 
 bool FilePath::IsDirectory() const {
+#ifdef _WIN32
+#if defined(_MSC_VER) && (_MSC_VER > 1310)
+	struct _stat64i32 statusFile;
+#else
+	struct _stat statusFile;
+#endif
+#else
 	struct stat statusFile;
-	if (stat(AsFileSystem(), &statusFile) != -1)
+#endif
+	if (stat(AsInternal(), &statusFile) != -1)
 #ifdef WIN32
 		return statusFile.st_mode & _S_IFDIR;
 #else
@@ -506,20 +510,37 @@ bool FilePath::IsDirectory() const {
 		return false;
 }
 
-bool FilePath::Matches(const char *pattern) const {
-	SString pat(pattern);
-	pat.substitute(' ', '\0');
-	SString nameCopy(Name().fileName);
-	nameCopy.lowercase();
+#ifdef _WIN32
+void Lowercase(GUI::gui_string &s) {
+	int chars = ::LCMapString(LOCALE_SYSTEM_DEFAULT, LCMAP_LOWERCASE, s.c_str(), s.size()+1, NULL, 0);
+	std::vector<wchar_t> vc(chars);
+	::LCMapString(LOCALE_SYSTEM_DEFAULT, LCMAP_LOWERCASE, s.c_str(), s.size()+1, &vc[0], chars);
+	s = &vc[0];
+}
+#endif
+
+static bool EndMatches(GUI::gui_string const &s, GUI::gui_string const &end) {
+	return (s.size() >= end.size()) &&
+		(std::equal(s.begin() + s.size() - end.size(), s.end(), end.begin()));
+}
+
+bool FilePath::Matches(const GUI::gui_char *pattern) const {
+	GUI::gui_string pat(pattern);
+	GUI::gui_string nameCopy(Name().fileName);
+#ifdef _WIN32
+	Lowercase(pat);
+	Lowercase(nameCopy);
+#endif
+	std::replace(pat.begin(), pat.end(), ' ', '\0');
 	size_t start = 0;
 	while (start < pat.length()) {
-		const char *patElement = pat.c_str() + start;
+		const GUI::gui_char *patElement = pat.c_str() + start;
 		if (patElement[0] == '*') {
-			if (nameCopy.endswith(patElement + 1)) {
+			if (EndMatches(nameCopy, patElement + 1)) {
 				return true;
 			}
 		} else {
-			if (nameCopy == SString(patElement).lowercase()) {
+			if (nameCopy == patElement) {
 				return true;
 			}
 		}
@@ -539,91 +560,12 @@ bool FilePath::Matches(const char *pattern) const {
  * @returns true on success, and the long path in @a longPath buffer,
  * false on failure, and copies the @a shortPath arg to the @a longPath buffer.
  */
-bool MakeLongPath(const char* shortPath, char* longPath) {
-	// when we have pfnGetLong, we assume it never changes as kernel32 is always loaded
-	static DWORD (STDAPICALLTYPE* pfnGetLong)(const char* lpszShortPath, char* lpszLongPath, DWORD cchBuffer) = NULL;
-	static bool kernelTried = FALSE;
-	bool ok = FALSE;
+static bool MakeLongPath(const GUI::gui_char* shortPath, GUI::gui_char* longPath) {
 
-	if (!kernelTried) {
-		HMODULE hModule;
-		kernelTried = true;
-		hModule = ::GetModuleHandleA("KERNEL32");
-		//assert(hModule != NULL); // must not call FreeLibrary on such handle
-
-		// attempt to get GetLongPathName (implemented in Win98/2000 only!)
-		(FARPROC&)pfnGetLong = ::GetProcAddress(hModule, "GetLongPathNameA");
-	}
-
-	// the kernel GetLongPathName proc is faster and (hopefully) more reliable
-	if (pfnGetLong != NULL) {
-		// call kernel proc
-		ok = (pfnGetLong)(shortPath, longPath, _MAX_PATH) != 0;
-	} else {
-		char short_path[_MAX_PATH];  // copy, so we can modify it
-		char* tok;
-
-		*longPath = '\0';
-
-		lstrcpyn(short_path, shortPath, _MAX_PATH);
-
-		for (;;) {
-			tok = strtok(short_path, "\\");
-			if (tok == NULL)
-				break;
-
-			if ((strlen(shortPath) > 3) &&
-			        (shortPath[0] == '\\') && (shortPath[1] == '\\')) {
-				// UNC, skip first seps
-				strcat(longPath, "\\\\");
-				strcat(longPath, tok);
-				strcat(longPath, "\\");
-
-				tok = strtok(NULL, "\\");
-				if (tok == NULL)
-					break;
-			}
-			strcat(longPath, tok);
-
-			bool isDir = false;
-
-			for (;;) {
-				WIN32_FIND_DATA fd;
-				HANDLE hfind;
-				char* tokend;
-
-				tok = strtok(NULL, "\\");
-				if (tok == NULL)
-					break;
-
-				strcat(longPath, "\\");
-				tokend = longPath + strlen(longPath);
-
-				// temporary add short component
-				strcpy(tokend, tok);
-
-				hfind = ::FindFirstFile(longPath, &fd);
-				if (hfind == INVALID_HANDLE_VALUE)
-					break;
-
-				isDir = (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
-
-				// finally add long component we got
-				strcpy(tokend, fd.cFileName);
-
-				::FindClose(hfind);
-			}
-			ok = tok == NULL;
-
-			if (ok && isDir)
-				strcat(longPath, "\\");
-
-			break;
-		}
-	}
+	bool ok = ::GetLongPathNameW(shortPath, longPath, _MAX_PATH) != 0;
 
 	if (!ok) {
-		lstrcpyn(longPath, shortPath, _MAX_PATH);
+		wcsncpy(longPath, shortPath, _MAX_PATH);
 	}
 	return ok;
 }
@@ -632,17 +574,17 @@ bool MakeLongPath(const char* shortPath, char* longPath) {
 void FilePath::FixName() {
 #ifdef WIN32
 	// Only used on Windows to use long file names and fix the case of file names
-	char longPath[_MAX_PATH];
+	GUI::gui_char longPath[_MAX_PATH];
 	// first try MakeLongPath which corrects the path and the case of filename too
-	if (MakeLongPath(AsFileSystem(), longPath)) {
+	if (MakeLongPath(AsInternal(), longPath)) {
 		Set(longPath);
 	} else {
 		// On Windows file comparison is done case insensitively so the user can
 		// enter scite.cxx and still open this file, SciTE.cxx. To ensure that the file
 		// is saved with correct capitalisation FindFirstFile is used to find out the
 		// real name of the file.
-		WIN32_FIND_DATA FindFileData;
-		HANDLE hFind = ::FindFirstFile(AsFileSystem(), &FindFileData);
+		WIN32_FIND_DATAW FindFileData;
+		HANDLE hFind = ::FindFirstFileW(AsInternal(), &FindFileData);
 		FilePath dir = Directory();
 		if (hFind != INVALID_HANDLE_VALUE) {	// FindFirstFile found the file
 			Set(dir, FindFileData.cFileName);
