@@ -6,15 +6,22 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <stdio.h>
+#include <time.h>
+
+#include <string>
 
 #include "Scintilla.h"
-#include "Accessor.h"
+
+#include "GUI.h"
+#include "SString.h"
+#include "FilePath.h"
+#include "StyleWriter.h"
 #include "Extender.h"
 #include "LuaExtension.h"
 
-#include "SString.h"
-#include "SciTEKeys.h"
 #include "IFaceTable.h"
+#include "SciTEKeys.h"
 
 extern "C" {
 #include "lua.h"
@@ -22,9 +29,7 @@ extern "C" {
 #include "lauxlib.h"
 }
 
-#include "Platform.h"
-
-#if PLAT_WIN
+#if !defined(__unix__)
 
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
@@ -720,7 +725,9 @@ static bool call_function(lua_State *L, int nargs, bool ignoreFunctionReturnValu
 	if (L) {
 		int traceback = 0;
 		if (tracebackEnabled) {
-			lua_getglobal(L, "print");
+			lua_getglobal(L, "debug");
+			lua_getfield(L, -1, "traceback");
+			lua_remove(L, -2);
 			if (lua_isfunction(L, -1)) {
 				traceback = lua_gettop(L) - nargs - 1;
 				lua_insert(L, traceback);
@@ -807,7 +814,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 
 	int arg = 2;
 
-	long params[2] = {0,0};
+	sptr_t params[2] = {0,0};
 
 	char *stringResult = 0;
 	bool needStringResult = false;
@@ -815,8 +822,8 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	int loopParamCount = 2;
 
 	if (func.paramType[0] == iface_length && func.paramType[1] == iface_string) {
-		params[0] = static_cast<long>(lua_strlen(L, arg));
-		params[1] = reinterpret_cast<long>(params[0] ? lua_tostring(L, arg) : "");
+		params[0] = lua_strlen(L, arg);
+		params[1] = reinterpret_cast<sptr_t>(params[0] ? lua_tostring(L, arg) : "");
 		loopParamCount = 0;
 	} else if (func.paramType[1] == iface_stringresult) {
 		needStringResult = true;
@@ -832,7 +839,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	for (int i=0; i<loopParamCount; ++i) {
 		if (func.paramType[i] == iface_string) {
 			const char *s = lua_tostring(L, arg++);
-			params[i] = reinterpret_cast<long>(s ? s : "");
+			params[i] = reinterpret_cast<sptr_t>(s ? s : "");
 		} else if (func.paramType[i] == iface_keymod) {
 			int keycode = static_cast<int>(luaL_checknumber(L, arg++)) & 0xFFFF;
 			int modifiers = static_cast<int>(luaL_checknumber(L, arg++)) & (SCMOD_SHIFT|SCMOD_CTRL|SCMOD_ALT);
@@ -851,7 +858,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 			stringResult = new char[stringResultLen+1];
 			if (stringResult) {
 				stringResult[stringResultLen]='\0';
-				params[1] = reinterpret_cast<long>(stringResult);
+				params[1] = reinterpret_cast<sptr_t>(stringResult);
 			} else {
 				raise_error(L, "String result buffer allocation failed");
 				return 0;
@@ -871,7 +878,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	// - numeric return type gets returned to lua as a number (following the stringresult)
 	// - other return types e.g. void get dropped.
 
-	int result = host->Send(p, func.value, reinterpret_cast<uptr_t&>(params[0]), reinterpret_cast<sptr_t&>(params[1]));
+	int result = host->Send(p, func.value, params[0], params[1]);
 
 	int resultCount = 0;
 
@@ -1364,9 +1371,12 @@ static bool InitGlobalScope(bool checkProperties, bool forceReload = false) {
 		// that should be blocked during startup, e.g. the ones that allow
 		// you to add or switch buffers?
 
-		luaL_loadfile(luaState, startupScript);
-		if (!call_function(luaState, 0, true)) {
-			host->Trace(">Lua: error occurred while loading startup script\n");
+		FilePath fpTest(GUI::StringFromUTF8(startupScript));
+		if (fpTest.Exists()) {
+			luaL_loadfile(luaState, startupScript);
+			if (!call_function(luaState, 0, true)) {
+				host->Trace(">Lua: error occurred while loading startup script\n");
+			}
 		}
 	}
 
@@ -1408,10 +1418,8 @@ bool LuaExtension::Finalise() {
 	// The rest don't strictly need to be cleared since they
 	// are never accessed except when luaState and host are set
 
-	if (startupScript) {
-		delete [] startupScript;
-		startupScript = NULL;
-	}
+	delete [] startupScript;
+	startupScript = NULL;
 
 	return false;
 }
@@ -1625,7 +1633,7 @@ struct StylingContext {
 	unsigned int startPos;
 	int lengthDoc;
 	int initStyle;
-	Accessor *styler;
+	StyleWriter *styler;
 
 	unsigned int endPos;
 	unsigned int endDoc;
@@ -1653,13 +1661,6 @@ struct StylingContext {
 		styler->ColourTo(end, state);
 	}
 
-	static int PropertyInt(lua_State *L) {
-		StylingContext *context = Context(L);
-		const char *s = luaL_checkstring(L, 2);
-		lua_pushnumber(L, context->styler->GetPropertyInt(s));
-		return 1;
-	}
-
 	static int Line(lua_State *L) {
 		StylingContext *context = Context(L);
 		int position = luaL_checkint(L, 2);
@@ -1751,23 +1752,23 @@ struct StylingContext {
 					(currentPos >= endPos);
 	}
 
-	void StartStyling(unsigned int startPos, unsigned int length, int initStyle) {
+	void StartStyling(unsigned int startPos_, unsigned int length, int initStyle_) {
 		endDoc = styler->Length();
-		endPos = startPos + length;
+		endPos = startPos_ + length;
 		if (endPos == endDoc)
 			endPos = endDoc + 1;
-		currentPos = startPos;
+		currentPos = startPos_;
 		atLineStart = true;
 		atLineEnd = false;
-		state = initStyle;
+		state = initStyle_;
 		cursorPos = 0;
 		lenCurrent = 0;
 		lenNext = 0;
 		memcpy(cursor[0], "\0\0\0\0\0\0\0\0", 8);
 		memcpy(cursor[1], "\0\0\0\0\0\0\0\0", 8);
 		memcpy(cursor[2], "\0\0\0\0\0\0\0\0", 8);
-		styler->StartAt(startPos, static_cast<char>(0xff));
-		styler->StartSegment(startPos);
+		styler->StartAt(startPos_, static_cast<char>(0xffu));
+		styler->StartSegment(startPos_);
 
 		GetNextChar();
 		cursorPos++;
@@ -1886,12 +1887,13 @@ struct StylingContext {
 		int len = end - start + 1;
 		if (len <= 0)
 			len = 1;
-		char *sReturn = new char[len];
+		char *sReturn = new char[len+1];
 		for (int i = 0; i < len; i++) {
 			sReturn[i] = context->styler->SafeGetCharAt(start + i);
 		}
 		sReturn[len] = '\0';
 		lua_pushstring(L, sReturn);
+		delete []sReturn;
 		return 1;
 	}
 
@@ -1918,7 +1920,7 @@ struct StylingContext {
 	}
 };
 
-bool LuaExtension::OnStyle(unsigned int startPos, int lengthDoc, int initStyle, Accessor *styler) {
+bool LuaExtension::OnStyle(unsigned int startPos, int lengthDoc, int initStyle, StyleWriter *styler) {
 	bool handled = false;
 	if (luaState) {
 		lua_getglobal(luaState, "OnStyle");
@@ -1946,10 +1948,11 @@ bool LuaExtension::OnStyle(unsigned int startPos, int lengthDoc, int initStyle,
 			lua_settable(luaState, -3);
 
 			lua_pushstring(luaState, "language");
-			lua_pushstring(luaState, host->Property("Language"));
+			char *lang = host->Property("Language");
+			lua_pushstring(luaState, lang);
+			delete []lang;
 			lua_settable(luaState, -3);
 
-			sc.PushMethod(luaState, StylingContext::PropertyInt, "PropertyInt");
 			sc.PushMethod(luaState, StylingContext::Line, "Line");
 			sc.PushMethod(luaState, StylingContext::CharAt, "CharAt");
 			sc.PushMethod(luaState, StylingContext::StyleAt, "StyleAt");
