@@ -19,16 +19,17 @@
 
 #include <string>
 #include <map>
+#include <sstream>
 
-#include "Platform.h"
-
-#if PLAT_GTK
+#if defined(GTK)
 
 #include <unistd.h>
 
 #endif
 
-#include "PropSet.h"
+#include "Scintilla.h"
+
+#include "GUI.h"
 
 #include "SString.h"
 #include "FilePath.h"
@@ -102,6 +103,20 @@ void PropSetFile::SetMultiple(const char *s) {
 	Set(s);
 }
 
+bool PropSetFile::Exists(const char *key) const {
+	mapss::const_iterator keyPos = props.find(std::string(key));
+	if (keyPos != props.end()) {
+		return true;
+	} else {
+		if (superPS) {
+			// Failed here, so try in base property set
+			return superPS->Exists(key);
+		} else {
+			return false;
+		}
+	}
+}
+
 SString PropSetFile::Get(const char *key) const {
 	mapss::const_iterator keyPos = props.find(std::string(key));
 	if (keyPos != props.end()) {
@@ -207,7 +222,7 @@ void PropSetFile::Clear() {
 
 char *PropSetFile::ToString() const {
 	std::string sval;
-	for (mapss::const_iterator it=props.begin(); it != props.end(); it++) {
+	for (mapss::const_iterator it=props.begin(); it != props.end(); ++it) {
 		sval += it->first;
 		sval += "=";
 		sval += it->second;
@@ -274,7 +289,7 @@ bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, FilePath direc
 	} else if (isPrefix(lineBuffer, "import ") && directoryForImports.IsSet()) {
 		SString importName(lineBuffer + strlen("import") + 1);
 		importName += ".properties";
-		FilePath importPath(directoryForImports, FilePath(importName.c_str()));
+		FilePath importPath(directoryForImports, FilePath(GUI::StringFromUTF8(importName.c_str())));
 		if (Read(importPath, directoryForImports, imports, sizeImports)) {
 			if (imports) {
 				for (int i = 0; i < sizeImports; i++) {
@@ -408,7 +423,7 @@ SString PropSetFile::GetWildUsingStart(const PropSetFile &psStart, const char *k
 		if (0 == strcmp(it->first.c_str(), keybase)) {
 			return SString(it->second.c_str());
 		}
-		it++;
+		++it;
 	}
 	if (superPS) {
 		// Failed here, so try in super property set
@@ -461,7 +476,7 @@ bool PropSetFile::GetFirst(const char *&key, const char *&val) {
 	if (it != props.end()) {
 		key = it->first.c_str();
 		val = it->second.c_str();
-		it++;
+		++it;
 		if (it != props.end()) {
 			enumnext = it->first; // GetNext will begin here ...
 		} else {
@@ -481,7 +496,7 @@ bool PropSetFile::GetNext(const char *&key, const char *&val) {
 	if (it != props.end()) {
 		key = it->first.c_str();
 		val = it->second.c_str();
-		it++;
+		++it;
 		if (it != props.end()) {
 			enumnext = it->first; // GetNext will begin here ...
 		} else {
@@ -528,6 +543,13 @@ SString::SString(int i) : sizeGrowth(sizeGrowthDefault) {
 	sSize = sLen = (s) ? strlen(s) : 0;
 }
 
+SString::SString(size_t i) : sizeGrowth(sizeGrowthDefault) {
+	std::ostringstream strstrm;
+	strstrm << i;
+	s = StringAllocate(strstrm.str().c_str());
+	sSize = sLen = (s) ? strlen(s) : 0;
+}
+
 SString::SString(double d, int precision) : sizeGrowth(sizeGrowthDefault) {
 	char number[32];
 	sprintf(number, "%.*f", precision, d);
@@ -559,10 +581,12 @@ SString &SString::assign(const char *sOther, lenpos_t sSize_) {
 		sSize_ = strlen(sOther);
 	}
 	if (sSize > 0 && sSize_ <= sSize) {	// Does not allocate new buffer if the current is big enough
-		if (s && sSize_) {
-			memcpy(s, sOther, sSize_);
+		if (s) {
+			if (sSize_) {
+				memcpy(s, sOther, sSize_);
+			}
+			s[sSize_] = '\0';
 		}
-		s[sSize_] = '\0';
 		sLen = sSize_;
 	} else {
 		delete []s;
