@@ -10,7 +10,7 @@
 
 typedef std::map<std::string, std::string> mapss;
 
-class PropSetFile : public PropertyGet {
+class PropSetFile {
 	bool lowerKeys;
 	SString GetWildUsingStart(const PropSetFile &psStart, const char *keybase, const char *filename);
 	static bool caseSensitiveFilenames;
@@ -24,6 +24,7 @@ public:
 	void Set(const char *keyVal);
 	void Unset(const char *key, int lenKey=-1);
 	void SetMultiple(const char *s);
+	bool Exists(const char *key) const;
 	SString Get(const char *key) const;
 	SString GetExpanded(const char *key) const;
 	SString Expand(const char *withVars, int maxExpands=100) const;
