@@ -2,16 +2,14 @@
 /** @file SciTEBase.h
  ** Definition of platform independent base class of editor.
  **/
-// Copyright 1998-2003 by Neil Hodgson <neilh@scintilla.org>
+// Copyright 1998-2010 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-extern const char appName[];
+extern const GUI::gui_char appName[];
 
-extern const char propUserFileName[];
-extern const char propGlobalFileName[];
-extern const char propAbbrevFileName[];
-
-extern const char menuAccessIndicator[];
+extern const GUI::gui_char propUserFileName[];
+extern const GUI::gui_char propGlobalFileName[];
+extern const GUI::gui_char propAbbrevFileName[];
 
 #ifdef WIN32
 #ifdef _MSC_VER
@@ -27,6 +25,23 @@ extern const char menuAccessIndicator[];
 
 #define ELEMENTS(a) (sizeof(a) / sizeof(a[0]))
 
+inline int Minimum(int a, int b) {
+	return (a < b) ? a : b;
+}
+
+inline int Maximum(int a, int b) {
+	return (a > b) ? a : b;
+}
+
+inline long LongFromTwoShorts(short a,short b) {
+	return (a) | ((b) << 16);
+}
+
+typedef long Colour;
+inline Colour ColourRGB(unsigned int red, unsigned int green, unsigned int blue) {
+	return red | (green << 8) | (blue << 16);
+}
+
 /**
  * The order of menus on Windows - the Buffers menu may not be present
  * and there is a Help menu at the end.
@@ -37,63 +52,6 @@ enum {
     menuHelp = 8
 };
 
-/**
- * This is a fixed length list of strings suitable for display in combo boxes
- * as a memory of user entries.
- */
-template < int sz >
-class EntryMemory {
-	SString entries[sz];
-public:
-	void Insert(const SString &s) {
-		for (int i = 0; i < sz; i++) {
-			if (entries[i] == s) {
-				for (int j = i; j > 0; j--) {
-					entries[j] = entries[j - 1];
-				}
-				entries[0] = s;
-				return;
-			}
-		}
-		for (int k = sz - 1; k > 0; k--) {
-			entries[k] = entries[k - 1];
-		}
-		entries[0] = s;
-	}
-	void AppendIfNotPresent(const SString &s) {
-		for (int i = 0; i < sz; i++) {
-			if (entries[i] == s) {
-				return;
-			}
-			if (0 == entries[i].length()) {
-				entries[i] = s;
-				return;
-			}
-		}
-	}
-	void AppendList(const SString &s, char sep = '|') {
-		int start = 0;
-		int end = 0;
-		while (s[end] != '\0') {
-			while ((s[end] != sep) && (s[end] != '\0'))
-				++end;
-			AppendIfNotPresent(SString(s.c_str(), start, end));
-			start = end + 1;
-			end = start;
-		}
-	}
-	int Length() const {
-		int len = 0;
-		for (int i = 0; i < sz; i++)
-			if (entries[i].length())
-				len++;
-		return len;
-	}
-	SString At(int n) const {
-		return entries[n];
-	}
-};
-
 class RecentFile : public FilePath {
 public:
 	Sci_CharacterRange selection;
@@ -117,85 +75,6 @@ enum UniMode {
     uniCookie = 4
 };
 
-// State of folding in a given document, remembers lines folded.
-class FoldState {
-private:
-	int *lines;
-	int size;
-	int fill;
-
-	void CopyFrom(const FoldState& b) {
-		Alloc(b.size);
-		memcpy(lines, b.lines, size*sizeof(int));
-		fill = b.fill;
-	}
-
-public:
-	FoldState() {
-		lines = 0;
-		size = 0;
-		fill = 0;
-	}
-
-	FoldState &operator=(const FoldState &b) {
-		if (this != &b) {
-			CopyFrom(b);
-		}
-		return *this;
-	}
-
-	FoldState(const FoldState &b) {
-		lines = 0;
-		size = 0;
-		fill = 0;
-
-		CopyFrom(b);
-	}
-
-	void Alloc(int s) {
-		//assert(s>0);
-		//assert(size==0);
-
-		delete []lines;
-		lines = new int[s];
-		size = s;
-		fill = 0;
-
-		//assert(lines && size>0);
-	}
-
-	void Clear() {
-
-		delete []lines;
-		lines = 0;
-
-		size = 0;
-		fill = 0;
-	}
-
-	virtual ~FoldState() {
-		Clear();
-	}
-
-	void Append(int line) {
-		//assert(fill<size);
-		lines[fill] = line;
-		fill++;
-	}
-
-	int Folds() const {
-		return size;
-	}
-
-	int Line(int fold) const {
-		if (fold >= size) {
-			return 0;
-		} else {
-			return lines[fold];
-		}
-	}
-};
-
 class Buffer : public RecentFile {
 public:
 	sptr_t doc;
@@ -206,7 +85,7 @@ public:
 	time_t fileModLastAsk;
 	enum { fmNone, fmMarked, fmModified} findMarks;
 	SString overrideExtension;	///< User has chosen to use a particular language
-	FoldState foldState;
+	std::vector<int> foldState;
 	Buffer() :
 			RecentFile(), doc(0), isDirty(false), useMonoFont(false),
 			unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), findMarks(fmNone), foldState() {}
@@ -220,7 +99,7 @@ public:
 		fileModLastAsk = 0;
 		findMarks = fmNone;
 		overrideExtension = "";
-		foldState.Clear();
+		foldState.clear();
 	}
 
 	void SetTimeFromFile() {
@@ -245,7 +124,7 @@ public:
 	int Add();
 	int GetDocumentByName(FilePath filename, bool excludeCurrent=false);
 	void RemoveCurrent();
-	int Current();
+	int Current() const;
 	Buffer *CurrentBuffer();
 	void SetCurrent(int index);
 	int StackNext();
@@ -271,8 +150,6 @@ public:
 	SString extension;
 };
 
-typedef EntryMemory < 10 > ComboMemory;
-
 enum {
     heightTools = 24,
     heightTab = 24,
@@ -326,25 +203,78 @@ public:
 struct StyleAndWords {
 	int styleNumber;
 	SString words;
+	StyleAndWords() : styleNumber(0) {
+	}
 	bool IsEmpty() { return words.length() == 0; }
 	bool IsSingleChar() { return words.length() == 1; }
 };
 
-class Localization : public PropSetFile {
+class Localization : public PropSetFile, public ILocalize {
 	SString missing;
 public:
 	bool read;
 	Localization() : PropSetFile(true), read(false) {
 	}
-	SString Text(const char *s, bool retainIfNotFound=true);
+	GUI::gui_string Text(const char *s, bool retainIfNotFound=true);
 	void SetMissing(const SString &missing_) {
 		missing = missing_;
 	}
 };
 
-class SciTEBase : public ExtensionAPI {
+// Interface between SciTE and dialogs and strips for find and replace
+class Searcher {
+public:
+	SString findWhat;
+	SString replaceWhat;
+
+	bool wholeWord;
+	bool matchCase;
+	bool regExp;
+	bool unSlash;
+	bool wrapFind;
+	bool reverseFind;
+
+	bool replacing;
+	bool havefound;
+	bool findInStyle;
+	int findStyle;
+	bool closeFind;
+	ComboMemory memFinds;
+	ComboMemory memReplaces;
+
+	bool focusOnReplace;
+
+	Searcher();
+
+	virtual void SetFind(const char *sFind) = 0;
+	virtual bool FindHasText() const = 0;
+	virtual void SetReplace(const char *sReplace) = 0;
+	virtual void MoveBack(int distance) = 0;
+	virtual void ScrollEditorIfNeeded() = 0;
+
+	virtual int FindNext(bool reverseDirection, bool showWarnings = true) = 0;
+	virtual int MarkAll() = 0;
+	virtual int ReplaceAll(bool inSelection) = 0;
+	virtual void ReplaceOnce() = 0;
+	virtual void UIClosed() = 0;
+	virtual void UIHasFocus() = 0;
+	bool &FlagFromCmd(int cmd);
+};
+
+class SearchUI {
 protected:
-	SString windowName;
+	Searcher *pSearcher;
+public:
+	SearchUI() : pSearcher(0) {
+	}
+	void SetSearcher(Searcher *pSearcher_) {
+		pSearcher = pSearcher_;
+	}
+};
+
+class SciTEBase : public ExtensionAPI, public Searcher {
+protected:
+	GUI::gui_string windowName;
 	FilePath filePath;
 	FilePath dirNameAtExecute;
 	FilePath dirNameForExecute;
@@ -357,26 +287,10 @@ protected:
 	FilePath importFiles[importMax];
 	enum { importCmdID = IDM_IMPORT };
 
-	SString findWhat;
-	SString replaceWhat;
 	enum { indicatorMatch = INDIC_CONTAINER };
 	enum { markerBookmark = 1 };
-	Window wFindIncrement;
-	bool replacing;
-	bool havefound;
-	bool matchCase;
-	bool wholeWord;
-	bool reverseFind;
-	bool regExp;
-	bool wrapFind;
-	bool unSlash;
-	bool findInStyle;
-	int findStyle;
-	ComboMemory memFinds;
-	ComboMemory memReplaces;
 	ComboMemory memFiles;
 	ComboMemory memDirectory;
-	enum { maxParam = 4 };
 	SString parameterisedCommand;
 	char abbrevInsert[200];
 
@@ -392,6 +306,7 @@ protected:
 	int characterSet;
 	SString language;
 	int lexLanguage;
+	int lexLPeg;
 	StringList apis;
 	SString apisFileNames;
 	SString functionDefinition;
@@ -410,19 +325,15 @@ protected:
 	StringList preprocCondMiddle;	///< List of preprocessor conditional middle keywords (in C: else elif)
 	StringList preprocCondEnd;	///< List of preprocessor conditional end keywords (in C: endif)
 
-	Window wSciTE;  ///< Contains wToolBar, wTabBar, wContent, and wStatusBar
-	Window wContent;    ///< Contains wEditor and wOutput
-	Window wEditor;
-	Window wOutput;
-	Window wIncrement;
-	Window wToolBar;
-	Window wStatusBar;
-	Window wTabBar;
-	Menu popup;
-	SciFnDirect fnEditor;
-	long ptrEditor;
-	SciFnDirect fnOutput;
-	long ptrOutput;
+	GUI::Window wSciTE;  ///< Contains wToolBar, wTabBar, wContent, and wStatusBar
+	GUI::Window wContent;    ///< Contains wEditor and wOutput
+	GUI::ScintillaWindow wEditor;
+	GUI::ScintillaWindow wOutput;
+	GUI::Window wIncrement;
+	GUI::Window wToolBar;
+	GUI::Window wStatusBar;
+	GUI::Window wTabBar;
+	GUI::Menu popup;
 	bool tbVisible;
 	bool tabVisible;
 	bool tabHideOne; // Hide tab bar if one buffer is opened only
@@ -448,11 +359,10 @@ protected:
 	enum { toolMax = 50 };
 	Extension *extender;
 	bool needReadProperties;
-	bool allowAlpha;
 
 	int heightOutput;
 	int heightOutputStartDrag;
-	Point ptStartDrag;
+	GUI::Point ptStartDrag;
 	bool capturedMouse;
 	int previousHeightOutput;
 	bool firstPropertiesRead;
@@ -545,7 +455,7 @@ protected:
 	void PrevInStack();
 	void EndStackedTabbing();
 
-	virtual void TabInsert(int index, char *title) = 0;
+	virtual void TabInsert(int index, const GUI::gui_char *title) = 0;
 	virtual void TabSelect(int index) = 0;
 	virtual void RemoveAllTabs() = 0;
 	void ShiftTab(int indexFrom, int indexTo);
@@ -557,21 +467,15 @@ protected:
 	void ReadLocalPropFile();
 	void ReadDirectoryPropFile();
 
-	sptr_t SendEditor(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	sptr_t SendEditorString(unsigned int msg, uptr_t wParam, const char *s);
-	sptr_t SendOutput(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	sptr_t SendOutputString(unsigned int msg, uptr_t wParam, const char *s);
-	sptr_t SendFocused(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	sptr_t SendPane(int destination, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	sptr_t SendWindow(Window &w, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	void SendChildren(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	sptr_t SendOutputEx(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0, bool direct = true);
+	sptr_t CallFocused(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
+	sptr_t CallPane(int destination, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
+	void CallChildren(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	SString GetTranslationToAbout(const char * const propname, bool retainIfNotFound = true);
 	int LengthDocument();
 	int GetCaretInLine();
 	void GetLine(char *text, int sizeText, int line = -1);
 	SString GetLine(int line = -1);
-	void GetRange(Window &win, int start, int end, char *text);
+	void GetRange(GUI::ScintillaWindow &win, int start, int end, char *text);
 	int IsLinePreprocessorCondition(char *line);
 	bool FindMatchingPreprocessorCondition(int &curLine, int direction, int condEnd1, int condEnd2);
 	bool FindMatchingPreprocCondPosition(bool isForward, int &mppcAtCaret, int &mppcMatch);
@@ -587,11 +491,11 @@ protected:
 	void ClearDocument();
 	void CreateBuffers();
 	void InitialiseBuffers();
-	FilePath UserFilePath(const char *name);
-	void LoadSessionFile(const char *sessionName);
+	FilePath UserFilePath(const GUI::gui_char *name);
+	void LoadSessionFile(const GUI::gui_char *sessionName);
 	void RestoreRecentMenu();
 	void RestoreSession();
-	void SaveSessionFile(const char *sessionName);
+	void SaveSessionFile(const GUI::gui_char *sessionName);
 	virtual void GetWindowPosition(int *left, int *top, int *width, int *height, int *maximize) = 0;
 	void SetIndentSettings();
 	void SetEol();
@@ -599,16 +503,16 @@ protected:
 	void RestoreState(const Buffer &buffer);
 	void Close(bool updateUI = true, bool loadingSession = false, bool makingRoomForNew = false);
 	bool IsAbsolutePath(const char *path);
-	bool Exists(const char *dir, const char *path, FilePath *resultPath);
+	bool Exists(const GUI::gui_char *dir, const GUI::gui_char *path, FilePath *resultPath);
 	void DiscoverEOLSetting();
 	void DiscoverIndentSetting();
 	SString DiscoverLanguage(const char *buf, size_t length);
 	void OpenFile(int fileSize, bool suppressMessage);
-	virtual void OpenUriList(const char *) {};
-	virtual bool OpenDialog(FilePath directory, const char *filter) = 0;
+	virtual void OpenUriList(const char *) {}
+	virtual bool OpenDialog(FilePath directory, const GUI::gui_char *filter) = 0;
 	virtual bool SaveAsDialog() = 0;
-	virtual void LoadSessionDialog() { };
-	virtual void SaveSessionDialog() { };
+	virtual void LoadSessionDialog() {}
+	virtual void SaveSessionDialog() {}
 	void CountLineEnds(int &linesCR, int &linesLF, int &linesCRLF);
 	enum OpenFlags {
 	    ofNone = 0, 		// Default
@@ -617,7 +521,7 @@ protected:
 	    ofPreserveUndo = 4,	// Do not delete undo history
 	    ofQuiet = 8		// Avoid "Could not open file" message
 	};
-	virtual bool PreOpenCheck(const char *file);
+	virtual bool PreOpenCheck(const GUI::gui_char *file);
 	bool Open(FilePath file, OpenFlags of = ofNone);
 	bool OpenSelected();
 	void Revert();
@@ -625,9 +529,9 @@ protected:
 	int SaveIfUnsure(bool forceQuestion = false);
 	int SaveIfUnsureAll(bool forceQuestion = false);
 	int SaveIfUnsureForBuilt();
-	void SaveIfNotOpen(const FilePath &destFile, bool fixCase);
+	bool SaveIfNotOpen(const FilePath &destFile, bool fixCase);
 	bool Save();
-	void SaveAs(const char *file, bool fixCase);
+	void SaveAs(const GUI::gui_char *file, bool fixCase);
 	virtual void SaveACopy() = 0;
 	void SaveToHTML(FilePath saveName);
 	void StripTrailingSpaces();
@@ -652,16 +556,16 @@ protected:
 	FilePath GetAbbrevPropertiesFileName();
 	void OpenProperties(int propsFile);
 	int GetMenuCommandAsInt(SString commandName);
-	virtual void Print(bool) {};
-	virtual void PrintSetup() {};
+	virtual void Print(bool) {}
+	virtual void PrintSetup() {}
 	Sci_CharacterRange GetSelection();
 	void SetSelection(int anchor, int currentPos);
 	//	void SelectionExtend(char *sel, int len, char *notselchar);
 	void GetCTag(char *sel, int len);
-	SString GetRange(Window &win, int selStart, int selEnd);
-	virtual SString GetRangeInUIEncoding(Window &win, int selStart, int selEnd);
-	SString GetLine(Window &win, int line);
-	SString RangeExtendAndGrab(Window &wCurrent, int &selStart, int &selEnd,
+	SString GetRange(GUI::ScintillaWindow &win, int selStart, int selEnd);
+	virtual SString GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd);
+	SString GetLine(GUI::ScintillaWindow &win, int line);
+	SString RangeExtendAndGrab(GUI::ScintillaWindow &wCurrent, int &selStart, int &selEnd,
 	        bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	SString SelectionExtend(bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
 	void FindWordAtCaret(int &start, int &end);
@@ -672,9 +576,14 @@ protected:
 	void SelectionIntoFind(bool stripEol = true);
 	virtual SString EncodeString(const SString &s);
 	virtual void Find() = 0;
-	virtual int WindowMessageBox(Window &w, const SString &m, int style) = 0;
+	virtual int WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, int style) = 0;
 	virtual void FindMessageBox(const SString &msg, const SString *findItem = 0) = 0;
 	int FindInTarget(const char *findWhat, int lenFind, int startPosition, int endPosition);
+	virtual void SetFind(const char *sFind);
+	virtual bool FindHasText() const;
+	virtual void SetReplace(const char *sReplace);
+	virtual void MoveBack(int distance);
+	virtual void ScrollEditorIfNeeded();
 	int FindNext(bool reverseDirection, bool showWarnings = true);
 	virtual void FindIncrement() = 0;
 	int IncrementSearchMode();
@@ -684,6 +593,8 @@ protected:
 	int DoReplaceAll(bool inSelection); // returns number of replacements or negative value if error
 	int ReplaceAll(bool inSelection);
 	int ReplaceInBuffers();
+	virtual void UIClosed();
+	virtual void UIHasFocus();
 	virtual void DestroyFindReplace() = 0;
 	virtual void GoLineDialog() = 0;
 	virtual bool AbbrevDialog() = 0;
@@ -743,7 +654,8 @@ protected:
 	void CloseAllBuffers(bool loadingSession = false);
 	int SaveAllBuffers(bool forceQuestion, bool alwaysYes = false);
 	void SaveTitledBuffers();
-	virtual void CopyAsRTF() {};
+	virtual void CopyAsRTF() {}
+	virtual void CopyPath() {}
 	void SetLineNumberWidth();
 	void MenuCommand(int cmdID, int source = 0);
 	void FoldChanged(int line, int levelNow, int levelPrev);
@@ -776,7 +688,7 @@ protected:
 	virtual void SizeSubWindows() = 0;
 
 	virtual void SetMenuItem(int menuNumber, int position, int itemID,
-	        const char *text, const char *mnemonic = 0) = 0;
+		const GUI::gui_char *text, const GUI::gui_char *mnemonic = 0) = 0;
 	virtual void RedrawMenu() {}
 	virtual void DestroyMenuItem(int menuNumber, int itemID) = 0;
 	virtual void CheckAMenuItem(int wIDCheckItem, bool val) = 0;
@@ -784,7 +696,7 @@ protected:
 	virtual void CheckMenusClipboard();
 	virtual void CheckMenus();
 	virtual void AddToPopUp(const char *label, int cmd = 0, bool enabled = true) = 0;
-	void ContextMenu(Window wSource, Point pt, Window wCmd);
+	void ContextMenu(GUI::ScintillaWindow &wSource, GUI::Point pt, GUI::Window wCmd);
 
 	void DeleteFileStackMenu();
 	void SetFileStackMenu();
@@ -808,13 +720,13 @@ protected:
 
 	void AssignKey(int key, int mods, int cmd);
 	void ViewWhitespace(bool view);
-	void SetAboutMessage(WindowID wsci, const char *appTitle);
+	void SetAboutMessage(GUI::ScintillaWindow &wsci, const char *appTitle);
 	void SetImportMenu();
 	void ImportMenu(int pos);
 	void SetLanguageMenu();
 	void SetPropertiesInitial();
-	SString LocaliseMessage(const char *s, const char *param0 = 0,
-	        const char *param1 = 0, const char *param2 = 0);
+	GUI::gui_string LocaliseMessage(const char *s,
+		const GUI::gui_char *param0 = 0, const GUI::gui_char *param1 = 0, const GUI::gui_char *param2 = 0);
 	virtual void ReadLocalization();
 	SString GetFileNameProperty(const char *name);
 	virtual void ReadPropertiesInitial();
@@ -824,20 +736,20 @@ protected:
 	SString ExtensionFileName();
 	const char *GetNextPropItem(const char *pStart, char *pPropItem, int maxLen);
 	void ForwardPropertyToEditor(const char *key);
-	void DefineMarker(int marker, int markerType, ColourDesired fore, ColourDesired back);
+	void DefineMarker(int marker, int markerType, Colour fore, Colour back);
 	void ReadAPI(const SString &fileNameForExtension);
 	SString FindLanguageProperty(const char *pattern, const char *defaultValue = "");
 	virtual void ReadProperties();
-	void SetOneStyle(Window &win, int style, const StyleDefinition &sd);
-	void SetStyleFor(Window &win, const char *language);
+	void SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd);
+	void SetStyleFor(GUI::ScintillaWindow &win, const char *language);
 	void ReloadProperties();
 
 	void CheckReload();
 	void Activate(bool activeApp);
-	PRectangle GetClientRectangle();
+	GUI::Rectangle GetClientRectangle();
 	void Redraw();
 	int NormaliseSplit(int splitPos);
-	void MoveSplit(Point ptNewDrag);
+	void MoveSplit(GUI::Point ptNewDrag);
 
 	void UIAvailable();
 	void PerformOne(char *action);
@@ -849,7 +761,7 @@ protected:
 	void AskMacroList();
 	bool StartMacroList(const char *words);
 	void ContinueMacroList(const char *stxt);
-	bool ProcessCommandLine(SString &args, int phase);
+	bool ProcessCommandLine(GUI::gui_string &args, int phase);
 	virtual bool IsStdinBlocked();
 	void OpenFromStdin(bool UseOutputPane);
 	void OpenFilesFromStdin();
@@ -857,8 +769,8 @@ protected:
 	    grepNone = 0, grepWholeWord = 1, grepMatchCase = 2, grepStdOut = 4,
 	    grepDot = 8, grepBinary = 16
 	};
-	void GrepRecursive(GrepFlags gf, FilePath baseDir, const char *searchString, const char *fileTypes);
-	void InternalGrep(GrepFlags gf, const char *directory, const char *files, const char *search);
+	void GrepRecursive(GrepFlags gf, FilePath baseDir, const char *searchString, const GUI::gui_char *fileTypes);
+	void InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *files, const char *search);
 	void EnumProperties(const char *action);
 	void SendOneProperty(const char *kind, const char *key, const char *val);
 	void PropertyFromDirector(const char *arg);
@@ -884,11 +796,13 @@ protected:
 	bool islexerwordcharforsel(char ch);
 public:
 
+	enum { maxParam = 4 };
+
 	SciTEBase(Extension *ext = 0);
 	virtual ~SciTEBase();
 
 	void ProcessExecute();
-	WindowID GetID() { return wSciTE.GetID(); }
+	GUI::WindowID GetID() { return wSciTE.GetID(); }
 
 private:
 	// un-implemented copy-constructor and assignment operator
@@ -896,16 +810,10 @@ private:
 	void operator=(const SciTEBase&);
 };
 
-struct ScintillaFailure {
-	int status;
-	ScintillaFailure(int status_) : status(status_) {
-	}
-};
-
 /// Base size of file I/O operations.
 const int blockSize = 131072;
 
-#if PLAT_GTK
+#if defined(__unix__)
 // MessageBox
 #define MB_OK	(0L)
 #define MB_YESNO	(0x4L)
@@ -920,10 +828,8 @@ const int blockSize = 131072;
 
 int ControlIDOfCommand(unsigned long);
 void LowerCaseString(char *s);
-long ColourOfProperty(PropSetFile &props, const char *key, ColourDesired colourDefault);
-char *Slash(const char *s, bool quoteQuotes);
-unsigned int UnSlash(char *s);
-void WindowSetFocus(Window &w);
+long ColourOfProperty(PropSetFile &props, const char *key, Colour colourDefault);
+void WindowSetFocus(GUI::ScintillaWindow &w);
 
 inline bool isspacechar(unsigned char ch) {
     return (ch == ' ') || ((ch >= 0x09) && (ch <= 0x0d));
