@@ -14,16 +14,11 @@
 #endif
 
 #include <string>
+#include <vector>
 #include <map>
 
-#ifdef __BORLANDC__
-// Borland includes Windows.h for STL and defaults to different API number
-#ifdef _WIN32_WINNT
 #undef _WIN32_WINNT
-#endif
-#endif
-
-#define _WIN32_WINNT  0x0400
+#define _WIN32_WINNT  0x0500
 #ifdef _MSC_VER
 // windows.h, et al, use a lot of nameless struct/unions - can't fix it, so allow it
 #pragma warning(disable: 4201)
@@ -35,19 +30,17 @@
 #endif
 #include <commctrl.h>
 
-#include "Platform.h"
+#include "Scintilla.h"
 
-#include "PropSet.h"
+#include "GUI.h"
 #include "SString.h"
 #include "StringList.h"
-
-#include "Scintilla.h"
-#include "Accessor.h"
+#include "StringHelpers.h"
+#include "FilePath.h"
+#include "PropSetFile.h"
 #include "Extender.h"
 #include "DirectorExtension.h"
 #include "SciTE.h"
-#include "FilePath.h"
-#include "PropSetFile.h"
 #include "Mutex.h"
 #include "JobQueue.h"
 #include "SciTEBase.h"
@@ -66,7 +59,7 @@ static void SendDirector(const char *verb, const char *arg = 0) {
 		SString addressedMessage;
 		if (wDestination) {
 			addressedMessage += ":";
-			SString address(reinterpret_cast<int>(wDestination));
+			SString address(reinterpret_cast<size_t>(wDestination));
 			addressedMessage += address;
 			addressedMessage += ":";
 		} else {
@@ -92,7 +85,7 @@ static void SendDirector(const char *verb, const char *arg = 0) {
 }
 
 static void SendDirector(const char *verb, sptr_t arg) {
-	SString s(arg);
+	SString s(static_cast<size_t>(arg));
 	::SendDirector(verb, s.c_str());
 }
 
@@ -108,13 +101,12 @@ static void CheckEnvironment(ExtensionAPI *host) {
 			}
 			delete []director;
 		}
-		char number[32];
-		sprintf(number, "%0d", reinterpret_cast<int>(wReceiver));
-		host->SetProperty("WindowID", number);
+		SString sReceiver(reinterpret_cast<size_t>(wReceiver));
+		host->SetProperty("WindowID", sReceiver.c_str());
 	}
 }
 
-static char DirectorExtension_ClassName[] = "DirectorExtension";
+static TCHAR DirectorExtension_ClassName[] = TEXT("DirectorExtension");
 
 static LRESULT HandleCopyData(LPARAM lParam) {
 	COPYDATASTRUCT *pcds = reinterpret_cast<COPYDATASTRUCT *>(lParam);
@@ -162,7 +154,7 @@ DirectorExtension &DirectorExtension::Instance() {
 
 bool DirectorExtension::Initialise(ExtensionAPI *host_) {
 	host = host_;
-	SDI = ::RegisterWindowMessage("SciTEDirectorInterface");
+	SDI = ::RegisterWindowMessage(TEXT("SciTEDirectorInterface"));
 	HINSTANCE hInstance = reinterpret_cast<HINSTANCE>(
 	                          host->GetInstance());
 	DirectorExtension_Register(hInstance);
@@ -250,7 +242,7 @@ bool DirectorExtension::OnSavePointLeft() {
 	return false;
 }
 
-bool DirectorExtension::OnStyle(unsigned int, int, int, Accessor *) {
+bool DirectorExtension::OnStyle(unsigned int, int, int, StyleWriter *) {
 	return false;
 }
 
