@@ -15,7 +15,7 @@ void SciTEWin::SetFileProperties(
 
 	const int TEMP_LEN = 100;
 	char temp[TEMP_LEN];
-	HANDLE hf = ::CreateFile(filePath.AsFileSystem(), GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+	HANDLE hf = ::CreateFileW(filePath.AsInternal(), GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 	if (hf != INVALID_HANDLE_VALUE) {
 		FILETIME ft;
 		::GetFileTime(hf, NULL, NULL, &ft);
@@ -24,17 +24,17 @@ void SciTEWin::SetFileProperties(
 		::FileTimeToLocalFileTime(&ft, &lft);
 		SYSTEMTIME st;
 		::FileTimeToSystemTime(&lft, &st);
-		::GetTimeFormat(LOCALE_USER_DEFAULT,
+		::GetTimeFormatA(LOCALE_USER_DEFAULT,
 		                0, &st,
 		                NULL, temp, TEMP_LEN);
 		ps.Set("FileTime", temp);
 
-		::GetDateFormat(LOCALE_USER_DEFAULT,
+		::GetDateFormatA(LOCALE_USER_DEFAULT,
 		                DATE_SHORTDATE, &st,
 		                NULL, temp, TEMP_LEN);
 		ps.Set("FileDate", temp);
 
-		DWORD attr = ::GetFileAttributes(filePath.AsFileSystem());
+		DWORD attr = ::GetFileAttributesW(filePath.AsInternal());
 		SString fa;
 		if (attr & FILE_ATTRIBUTE_READONLY) {
 			fa += "R";
@@ -53,12 +53,12 @@ void SciTEWin::SetFileProperties(
 		ps.Set("FileAttr", "");
 	}
 
-	::GetDateFormat(LOCALE_USER_DEFAULT,
+	::GetDateFormatA(LOCALE_USER_DEFAULT,
 	                DATE_SHORTDATE, NULL,     	// Current date
 	                NULL, temp, TEMP_LEN);
 	ps.Set("CurrentDate", temp);
 
-	::GetTimeFormat(LOCALE_USER_DEFAULT,
+	::GetTimeFormatA(LOCALE_USER_DEFAULT,
 	                0, NULL,     	// Current time
 	                NULL, temp, TEMP_LEN);
 	ps.Set("CurrentTime", temp);
@@ -68,16 +68,17 @@ void SciTEWin::SetFileProperties(
  * Update the status bar text.
  */
 void SciTEWin::SetStatusBarText(const char *s) {
+	GUI::gui_string barText = GUI::StringFromUTF8(s);
 	::SendMessage(reinterpret_cast<HWND>(wStatusBar.GetID()),
-	              SB_SETTEXT, 0, reinterpret_cast<LPARAM>(s));
+	              SB_SETTEXT, 0, reinterpret_cast<LPARAM>(barText.c_str()));
 }
 
-void SciTEWin::TabInsert(int index, char *title) {
-	TCITEM tie;
+void SciTEWin::TabInsert(int index, const GUI::gui_char *title) {
+	TCITEMW tie;
 	tie.mask = TCIF_TEXT | TCIF_IMAGE;
 	tie.iImage = -1;
-	tie.pszText = title;
-	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_INSERTITEM, (WPARAM)index, (LPARAM)&tie);
+	tie.pszText = const_cast<GUI::gui_char *>(title);
+	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()), TCM_INSERTITEMW, (WPARAM)index, (LPARAM)&tie);
 }
 
 void SciTEWin::TabSelect(int index) {
@@ -96,7 +97,7 @@ void SciTEWin::Notify(SCNotification *notification) {
 	case TCN_SELCHANGE:
 		// Change of tab
 		if (notification->nmhdr.idFrom == IDM_TABWIN) {
-			int index = Platform::SendScintilla(wTabBar.GetID(), TCM_GETCURSEL, (WPARAM)0, (LPARAM)0);
+			int index = ::SendMessage(static_cast<HWND>(wTabBar.GetID()), TCM_GETCURSEL, (WPARAM)0, (LPARAM)0);
 			SetDocumentAt(index);
 			CheckReload();
 		}
@@ -106,9 +107,9 @@ void SciTEWin::Notify(SCNotification *notification) {
 		// Right click on a control
 		if (notification->nmhdr.idFrom == IDM_TABWIN) {
 
-			Point ptCursor;
+			GUI::Point ptCursor;
 			::GetCursorPos(reinterpret_cast<POINT *>(&ptCursor));
-			Point ptClient = ptCursor;
+			GUI::Point ptClient = ptCursor;
 			::ScreenToClient(reinterpret_cast<HWND>(wTabBar.GetID()),
 			                 reinterpret_cast<POINT *>(&ptClient));
 			TCHITTESTINFO info;
@@ -136,7 +137,7 @@ void SciTEWin::Notify(SCNotification *notification) {
 				SString prefix = "command.name.";
 				prefix += SString(item);
 				prefix += ".";
-				SString commandName = props.GetNewExpand(prefix.c_str(), filePath.AsInternal());
+				SString commandName = props.GetNewExpand(prefix.c_str(), filePath.AsUTF8().c_str());
 				if (commandName.length()) {
 					SString sMenuItem = commandName;
 					SString sMnemonic = "Ctrl+";
@@ -176,88 +177,88 @@ void SciTEWin::Notify(SCNotification *notification) {
 	case TTN_GETDISPINFO:
 		// Ask for tooltip text
 		{
-			static char ttt[MAX_PATH*2 + 1];
-			const char *ttext = 0;
-			NMTTDISPINFO *pDispInfo = (NMTTDISPINFO *)notification;
+			static GUI::gui_char ttt[MAX_PATH*2 + 1];
+			const GUI::gui_char *ttext = 0;
+			NMTTDISPINFOW *pDispInfo = (NMTTDISPINFOW *)notification;
 			// Toolbar tooltips
 			switch (notification->nmhdr.idFrom) {
 			case IDM_NEW:
-				ttext = "New";
+				ttext = GUI_TEXT("New");
 				break;
 			case IDM_OPEN:
-				ttext = "Open";
+				ttext = GUI_TEXT("Open");
 				break;
 			case IDM_SAVE:
-				ttext = "Save";
+				ttext = GUI_TEXT("Save");
 				break;
 			case IDM_CLOSE:
-				ttext = "Close";
+				ttext = GUI_TEXT("Close");
 				break;
 			case IDM_PRINT:
-				ttext = "Print";
+				ttext = GUI_TEXT("Print");
 				break;
 			case IDM_CUT:
-				ttext = "Cut";
+				ttext = GUI_TEXT("Cut");
 				break;
 			case IDM_COPY:
-				ttext = "Copy";
+				ttext = GUI_TEXT("Copy");
 				break;
 			case IDM_PASTE:
-				ttext = "Paste";
+				ttext = GUI_TEXT("Paste");
 				break;
 			case IDM_CLEAR:
-				ttext = "Delete";
+				ttext = GUI_TEXT("Delete");
 				break;
 			case IDM_UNDO:
-				ttext = "Undo";
+				ttext = GUI_TEXT("Undo");
 				break;
 			case IDM_REDO:
-				ttext = "Redo";
+				ttext = GUI_TEXT("Redo");
 				break;
 			case IDM_FIND:
-				ttext = "Find";
+				ttext = GUI_TEXT("Find");
 				break;
 			case IDM_REPLACE:
-				ttext = "Replace";
+				ttext = GUI_TEXT("Replace");
 				break;
 			case IDM_MACRORECORD:
-				ttext = "Record Macro";
+				ttext = GUI_TEXT("Record Macro");
 				break;
 			case IDM_MACROSTOPRECORD:
-				ttext = "Stop Recording";
+				ttext = GUI_TEXT("Stop Recording");
 				break;
 			case IDM_MACROPLAY:
-				ttext = "Run Macro";
+				ttext = GUI_TEXT("Run Macro");
 				break;
 			default: {
 					// notification->nmhdr.idFrom appears to be the buffer number for tabbar tooltips
-					Point ptCursor;
+					GUI::Point ptCursor;
 					::GetCursorPos(reinterpret_cast<POINT *>(&ptCursor));
-					Point ptClient = ptCursor;
+					GUI::Point ptClient = ptCursor;
 					::ScreenToClient(reinterpret_cast<HWND>(wTabBar.GetID()),
 					                 reinterpret_cast<POINT *>(&ptClient));
 					TCHITTESTINFO info;
 					info.pt.x = ptClient.x;
 					info.pt.y = ptClient.y;
-					int index = Platform::SendScintilla(wTabBar.GetID(), TCM_HITTEST, (WPARAM)0, (LPARAM) & info);
+					int index = ::SendMessage(static_cast<HWND>(wTabBar.GetID()), TCM_HITTEST, (WPARAM)0, (LPARAM) & info);
 					if (index >= 0) {
-						SString path = buffers.buffers[index].AsInternal();
+						GUI::gui_string path = buffers.buffers[index].AsInternal();
 						// Handle '&' characters in path, since they are interpreted in
 						// tooltips.
-						int amp = 0;
-						while ((amp = path.search("&", amp)) >= 0) {
-							path.insert(amp, "&");
+						size_t amp = 0;
+						while ((amp = path.find(GUI_TEXT("&"), amp)) != GUI::gui_string::npos) {
+							path.insert(amp, GUI_TEXT("&"));
 							amp += 2;
 						}
-						strcpy(ttt, path.c_str());
-						pDispInfo->lpszText = const_cast<char *>(ttt);
+						wcscpy(ttt, path.c_str());
+						pDispInfo->lpszText = const_cast<GUI::gui_char *>(ttt);
 					}
 				}
 				break;
 			}
 			if (ttext) {
-				SString localised = localiser.Text(ttext);
-				strcpy(ttt, localised.c_str());
+				GUI::gui_string localised = localiser.Text(GUI::UTF8FromString(ttext).c_str());
+				wcscpy(ttt, localised.c_str());
 				pDispInfo->lpszText = ttt;
 			}
 			break;
@@ -304,18 +305,18 @@ void SciTEWin::ActivateWindow(const char *) {
  * Resize the content windows, embedding the editor and output windows.
  */
 void SciTEWin::SizeContentWindows() {
-	PRectangle rcInternal = wContent.GetClientPosition();
+	GUI::Rectangle rcInternal = wContent.GetClientPosition();
 
 	int w = rcInternal.Width();
 	int h = rcInternal.Height();
 	heightOutput = NormaliseSplit(heightOutput);
 
 	if (splitVertical) {
-		wEditor.SetPosition(PRectangle(0, 0, w - heightOutput - heightBar, h));
-		wOutput.SetPosition(PRectangle(w - heightOutput, 0, w, h));
+		wEditor.SetPosition(GUI::Rectangle(0, 0, w - heightOutput - heightBar, h));
+		wOutput.SetPosition(GUI::Rectangle(w - heightOutput, 0, w, h));
 	} else {
-		wEditor.SetPosition(PRectangle(0, 0, w, h - heightOutput - heightBar));
-		wOutput.SetPosition(PRectangle(0, h - heightOutput, w, h));
+		wEditor.SetPosition(GUI::Rectangle(0, 0, w, h - heightOutput - heightBar));
+		wOutput.SetPosition(GUI::Rectangle(0, h - heightOutput, w, h));
 	}
 	wContent.InvalidateAll();
 }
@@ -324,11 +325,12 @@ void SciTEWin::SizeContentWindows() {
  * Resize the sub-windows, ie. the toolbar, tab bar, status bar. And call @a SizeContentWindows.
  */
 void SciTEWin::SizeSubWindows() {
-	PRectangle rcClient = wSciTE.GetClientPosition();
+	GUI::Rectangle rcClient = wSciTE.GetClientPosition();
 	bool showTab = false;
 
 	//::SendMessage(MainHWND(), WM_SETREDRAW, false, 0); // suppress flashing
 	visHeightTools = tbVisible ? heightTools : 0;
+	bands[bandTool].visible = tbVisible;
 
 	if (tabVisible) {	// ? hide one tab only
 		showTab = tabHideOne ?
@@ -336,60 +338,75 @@ void SciTEWin::SizeSubWindows() {
 		          true;
 	}
 
-	if (showTab) {
-		wTabBar.SetPosition(PRectangle(
+	bands[bandTab].visible = showTab;
+	if (showTab && tabMultiLine) {
+		wTabBar.SetPosition(GUI::Rectangle(
 			rcClient.left, rcClient.top + visHeightTools,
 			rcClient.right, rcClient.top + heightTab + visHeightTools));
-		int tabNb = ::SendMessage(reinterpret_cast<HWND>(
-			wTabBar.GetID()), TCM_GETROWCOUNT, 0, 0);
-		visHeightTab = ((tabNb - 1) * (heightTab - 6)) + heightTab;
-	} else {
-		visHeightTab = 0;
-	}
-	visHeightStatus = sbVisible ? heightStatus : 0;
-	visHeightEditor = rcClient.Height() - visHeightTools - visHeightStatus - visHeightTab;
-	if (visHeightEditor < 1) {
-		visHeightTools = 1;
-		visHeightStatus = 1;
-		visHeightTab = 1;
-		visHeightEditor = rcClient.Height() - visHeightTools - visHeightStatus - visHeightTab;
-	}
-	if (tbVisible) {
-		wToolBar.SetPosition(PRectangle(
-		                         rcClient.left, rcClient.top, rcClient.right, visHeightTools));
-		wToolBar.Show(true);
-	} else {
-		wToolBar.Show(false);
-		wToolBar.SetPosition(PRectangle(
-		                         rcClient.left, rcClient.top - 2, rcClient.Width(), 1));
-	}
-	if (showTab) {
-		wTabBar.SetPosition(PRectangle(
-		                        rcClient.left, rcClient.top + visHeightTools,
-		                        rcClient.right, rcClient.top + visHeightTab + visHeightTools));
-		wTabBar.Show(true);
-	} else {
-		wTabBar.Show(false);
-		wTabBar.SetPosition(PRectangle(
-		                        rcClient.left, rcClient.top - 2,
-		                        rcClient.Width(), 1));
-	}
-	if (sbVisible) {
-		wStatusBar.SetPosition(PRectangle(
-		                           rcClient.left, rcClient.top + visHeightTools + visHeightTab + visHeightEditor,
-		                           rcClient.right,
-		                           rcClient.top + visHeightTools + visHeightTab + visHeightEditor + visHeightStatus));
-		wStatusBar.Show(true);
-	} else {
-		wStatusBar.Show(false);
-		wStatusBar.SetPosition(PRectangle(
-		                           rcClient.left, rcClient.top - 2, rcClient.Width(), 1));
 	}
 
-	wContent.SetPosition(PRectangle(
-	                         rcClient.left, rcClient.top + visHeightTab + visHeightTools,
-	                         rcClient.right,
-	                         rcClient.top + visHeightTab + visHeightTools + visHeightEditor));
+	RECT r = { rcClient.left, 0, rcClient.right, 0 };
+	::SendMessage(reinterpret_cast<HWND>(wTabBar.GetID()),
+		TCM_ADJUSTRECT, TRUE, LPARAM(&r));
+	bands[bandTab].height = r.bottom - r.top - 4;
+
+	bands[bandSearch].visible = searchStrip.visible;
+	bands[bandFind].visible = findStrip.visible;
+	bands[bandReplace].visible = replaceStrip.visible;
+
+	GUI::Rectangle rcSB = wStatusBar.GetPosition();
+	bands[bandStatus].height = rcSB.Height() - 2;	// -2 hides a top border
+	bands[bandStatus].visible = sbVisible;
+
+	int heightContent = rcClient.Height();
+	if (heightContent <= 0)
+		heightContent = 1;
+
+	for (size_t i=0;i<bands.size();i++) {
+		if (bands[i].visible && !bands[i].expands)
+			heightContent -= bands[i].height;
+	}
+	if (heightContent <= 0) {
+		heightContent = rcClient.Height();
+		for (size_t i=0;i<bands.size();i++) {
+			if (i != bandContents)
+				bands[i].visible = false;
+		}
+	}
+	bands[bandContents].height = heightContent;
+
+	// May need to copy some values out to other variables
+
+	HDWP hdwp = BeginDeferWindowPos(10);
+
+	int yPos = rcClient.top;
+	for (size_t b=0; b<bands.size(); b++) {
+		if (bands[b].visible) {
+			GUI::Rectangle rcToSet(rcClient.left, yPos, rcClient.right, yPos + bands[b].height);
+			if (hdwp)
+				hdwp = ::DeferWindowPos(hdwp, reinterpret_cast<HWND>(bands[b].win.GetID()),
+				0, rcToSet.left, rcToSet.top, rcToSet.Width(), rcToSet.Height(),
+				SWP_NOZORDER|SWP_NOACTIVATE|SWP_SHOWWINDOW);
+			//bands[b].win.Show(true);
+			yPos += bands[b].height;
+		} else {
+			GUI::Rectangle rcToSet(rcClient.left, rcClient.top - 41, rcClient.Width(), rcClient.top - 40);
+			if (hdwp)
+				hdwp = ::DeferWindowPos(hdwp, reinterpret_cast<HWND>(bands[b].win.GetID()),
+				0, rcToSet.left, rcToSet.top, rcToSet.Width(), rcToSet.Height(),
+				SWP_NOZORDER|SWP_NOACTIVATE|SWP_HIDEWINDOW);
+			//bands[b].win.Show(false);
+			//bands[b].win.SetPosition();
+		}
+	}
+	if (hdwp)
+		::EndDeferWindowPos(hdwp);
+
+	visHeightTools = bands[bandTool].height;
+	visHeightTab = bands[bandTab].height;
+	visHeightEditor = bands[bandContents].height;
+	visHeightStatus = bands[bandStatus].height;
+
 	SizeContentWindows();
 	//::SendMessage(MainHWND(), WM_SETREDRAW, true, 0);
 	//::RedrawWindow(MainHWND(), NULL, NULL, RDW_INVALIDATE | RDW_ALLCHILDREN);
@@ -403,15 +420,15 @@ void SciTEWin::SizeSubWindows() {
 
 
 void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
-                           const char *text, const char *mnemonic) {
+                           const GUI::gui_char *text, const GUI::gui_char *mnemonic) {
 	// On Windows the menu items are modified if they already exist or are created
 	HMENU hmenu = ::GetSubMenu(::GetMenu(MainHWND()), menuNumber);
-	SString sTextMnemonic = text;
+	GUI::gui_string sTextMnemonic = text;
 	long keycode = 0;
 	if (mnemonic && *mnemonic) {
-		keycode = SciTEKeys::ParseKeyCode(mnemonic);
+		keycode = SciTEKeys::ParseKeyCode(GUI::UTF8FromString(mnemonic).c_str());
 		if (keycode) {
-			sTextMnemonic += "\t";
+			sTextMnemonic += GUI_TEXT("\t");
 			sTextMnemonic += LocaliseAccelerator(mnemonic, itemID);
 		}
 		// the keycode could be used to make a custom accelerator table
@@ -421,11 +438,11 @@ void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
 
 	if (::GetMenuState(hmenu, itemID, MF_BYCOMMAND) == 0xffffffff) {
 		if (text[0])
-			::InsertMenu(hmenu, position, MF_BYPOSITION, itemID, sTextMnemonic.c_str());
+			::InsertMenuW(hmenu, position, MF_BYPOSITION, itemID, sTextMnemonic.c_str());
 		else
-			::InsertMenu(hmenu, position, MF_BYPOSITION | MF_SEPARATOR, itemID, sTextMnemonic.c_str());
+			::InsertMenuW(hmenu, position, MF_BYPOSITION | MF_SEPARATOR, itemID, sTextMnemonic.c_str());
 	} else {
-		::ModifyMenu(hmenu, position, MF_BYCOMMAND, itemID, sTextMnemonic.c_str());
+		::ModifyMenuW(hmenu, position, MF_BYCOMMAND, itemID, sTextMnemonic.c_str());
 	}
 
 	if (itemID >= IDM_TOOLS && itemID < IDM_TOOLS + toolMax) {
@@ -465,7 +482,7 @@ void SciTEWin::CheckAMenuItem(int wIDCheckItem, bool val) {
 void EnableButton(HWND wTools, int id, bool enable) {
 	if (wTools) {
 		::SendMessage(wTools, TB_ENABLEBUTTON, id,
-	              Platform::LongFromTwoShorts(static_cast<short>(enable ? TRUE : FALSE), 0));
+	              LongFromTwoShorts(static_cast<short>(enable ? TRUE : FALSE), 0));
 	}
 }
 
@@ -479,9 +496,9 @@ void SciTEWin::EnableAMenuItem(int wIDCheckItem, bool val) {
 
 void SciTEWin::CheckMenus() {
 	SciTEBase::CheckMenus();
-	CheckMenuRadioItem(::GetMenu(MainHWND()), IDM_EOL_CRLF, IDM_EOL_LF,
-	                   SendEditor(SCI_GETEOLMODE) - SC_EOL_CRLF + IDM_EOL_CRLF, 0);
-	CheckMenuRadioItem(::GetMenu(MainHWND()), IDM_ENCODING_DEFAULT, IDM_ENCODING_UCOOKIE,
+	::CheckMenuRadioItem(::GetMenu(MainHWND()), IDM_EOL_CRLF, IDM_EOL_LF,
+	                   wEditor.Call(SCI_GETEOLMODE) - SC_EOL_CRLF + IDM_EOL_CRLF, 0);
+	::CheckMenuRadioItem(::GetMenu(MainHWND()), IDM_ENCODING_DEFAULT, IDM_ENCODING_UCOOKIE,
 	                   CurrentBuffer()->unicodeMode + IDM_ENCODING_DEFAULT, 0);
 }
 
@@ -557,7 +574,7 @@ void SciTEWin::MakeAccelerator(SString sAccelerator, ACCEL &Accel) {
 }
 
 //SString SciTEWin::LocaliseAccelerator(const char *pAccelerator, int cmd) {
-SString SciTEWin::LocaliseAccelerator(const char *pAccelerator, int) {
+GUI::gui_string SciTEWin::LocaliseAccelerator(const GUI::gui_char *pAccelerator, int) {
 #ifdef LOCALISE_ACCELERATORS_WORKED
 	SString translation = localiser.Text(pAccelerator, true);
 	int AccelCount = ::CopyAcceleratorTable(hAccTable, NULL, 0);
@@ -585,42 +602,39 @@ SString SciTEWin::LocaliseAccelerator(const char *pAccelerator, int) {
 
 void SciTEWin::LocaliseMenu(HMENU hmenu) {
 	for (int i = 0; i <= ::GetMenuItemCount(hmenu); i++) {
-		char buff[200];
+		GUI::gui_char buff[200];
 		buff[0] = '\0';
-		MENUITEMINFO mii;
+		MENUITEMINFOW mii;
 		memset(&mii, 0, sizeof(mii));
-		// Explicitly use the struct size for NT 4 as otherwise
-		// GetMenuItemInfo will fail on NT 4.
-		//mii.cbSize = sizeof(mii);
-		mii.cbSize = 44;
+		mii.cbSize = sizeof(mii);
 		mii.fMask = MIIM_CHECKMARKS | MIIM_DATA | MIIM_ID |
 		            MIIM_STATE | MIIM_SUBMENU | MIIM_TYPE;
 		mii.dwTypeData = buff;
 		mii.cch = sizeof(buff) - 1;
-		if (::GetMenuItemInfo(hmenu, i, TRUE, &mii)) {
+		if (::GetMenuItemInfoW(hmenu, i, TRUE, &mii)) {
 			if (mii.hSubMenu) {
 				LocaliseMenu(mii.hSubMenu);
 			}
 			if (mii.fType == MFT_STRING || mii.fType == MFT_RADIOCHECK) {
 				if (mii.dwTypeData) {
-					SString text(mii.dwTypeData);
-					SString accel(mii.dwTypeData);
+					GUI::gui_string text(mii.dwTypeData);
+					GUI::gui_string accel(mii.dwTypeData);
 					size_t len = text.length();
-					int tab = text.search("\t");
-					if (tab != -1) {
-						text.remove(tab, len - tab);
-						accel.remove(0, tab + 1);
+					size_t tab = text.find(GUI_TEXT("\t"));
+					if (tab != GUI::gui_string::npos) {
+						text.erase(tab, len - tab);
+						accel.erase(0, tab + 1);
 					} else {
-						accel = "";
+						accel = GUI_TEXT("");
 					}
-					text = localiser.Text(text.c_str(), true);
+					text = localiser.Text(GUI::UTF8FromString(text.c_str()).c_str(), true);
 					if (text.length()) {
-						if (accel != "") {
-							text += "\t";
+						if (accel != GUI_TEXT("")) {
+							text += GUI_TEXT("\t");
 							text += LocaliseAccelerator(accel.c_str(), mii.wID);
 						}
-						mii.dwTypeData = const_cast<char *>(text.c_str());
-						::SetMenuItemInfo(hmenu, i, TRUE, &mii);
+						mii.dwTypeData = const_cast<GUI::gui_char *>(text.c_str());
+						::SetMenuItemInfoW(hmenu, i, TRUE, &mii);
 					}
 				}
 			}
@@ -634,16 +648,16 @@ void SciTEWin::LocaliseMenus() {
 }
 
 void SciTEWin::LocaliseAccelerators() {
-	LocaliseAccelerator("Alt+1", IDM_BUFFER + 0);
-	LocaliseAccelerator("Alt+2", IDM_BUFFER + 1);
-	LocaliseAccelerator("Alt+3", IDM_BUFFER + 2);
-	LocaliseAccelerator("Alt+4", IDM_BUFFER + 3);
-	LocaliseAccelerator("Alt+5", IDM_BUFFER + 4);
-	LocaliseAccelerator("Alt+6", IDM_BUFFER + 5);
-	LocaliseAccelerator("Alt+7", IDM_BUFFER + 6);
-	LocaliseAccelerator("Alt+8", IDM_BUFFER + 7);
-	LocaliseAccelerator("Alt+9", IDM_BUFFER + 8);
-	LocaliseAccelerator("Alt+0", IDM_BUFFER + 9);
+	LocaliseAccelerator(GUI_TEXT("Alt+1"), IDM_BUFFER + 0);
+	LocaliseAccelerator(GUI_TEXT("Alt+2"), IDM_BUFFER + 1);
+	LocaliseAccelerator(GUI_TEXT("Alt+3"), IDM_BUFFER + 2);
+	LocaliseAccelerator(GUI_TEXT("Alt+4"), IDM_BUFFER + 3);
+	LocaliseAccelerator(GUI_TEXT("Alt+5"), IDM_BUFFER + 4);
+	LocaliseAccelerator(GUI_TEXT("Alt+6"), IDM_BUFFER + 5);
+	LocaliseAccelerator(GUI_TEXT("Alt+7"), IDM_BUFFER + 6);
+	LocaliseAccelerator(GUI_TEXT("Alt+8"), IDM_BUFFER + 7);
+	LocaliseAccelerator(GUI_TEXT("Alt+9"), IDM_BUFFER + 8);
+	LocaliseAccelerator(GUI_TEXT("Alt+0"), IDM_BUFFER + 9);
 
 	// todo read keymap from cfg
 	// AssignKey('Y', SCMOD_CTRL, SCI_LINECUT);
@@ -651,10 +665,10 @@ void SciTEWin::LocaliseAccelerators() {
 
 void SciTEWin::LocaliseControl(HWND w) {
 	char wtext[200];
-	if (::GetWindowText(w, wtext, sizeof(wtext))) {
-		SString text = localiser.Text(wtext, false);
+	if (::GetWindowTextA(w, wtext, sizeof(wtext))) {
+		GUI::gui_string text = localiser.Text(wtext, false);
 		if (text.length())
-			::SetWindowText(w, text.c_str());
+			::SetWindowTextW(w, text.c_str());
 	}
 }
 
@@ -712,7 +726,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 	switch (iMessage) {
 
 	case WM_LBUTTONDOWN: {
-			Point pt = Point::FromLong(lParam);
+			GUI::Point pt = PointFromLong(lParam);
 			TCHITTESTINFO thti;
 			thti.pt.x = pt.x;
 			thti.pt.y = pt.y;
@@ -733,7 +747,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 
 	case WM_MBUTTONDOWN: {
 			// Check if on tab bar
-			Point pt = Point::FromLong(lParam);
+			GUI::Point pt = PointFromLong(lParam);
 			TCHITTESTINFO thti;
 			thti.pt.x = pt.x;
 			thti.pt.y = pt.y;
@@ -752,7 +766,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 				::ReleaseCapture();
 				::SetCursor(::LoadCursor(NULL, IDC_ARROW));
 				st_bDragBegin = FALSE;
-				Point pt = Point::FromLong(lParam);
+				GUI::Point pt = PointFromLong(lParam);
 				TCHITTESTINFO thti;
 				thti.pt.x = pt.x;
 				thti.pt.y = pt.y;
@@ -786,7 +800,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 
 	case WM_MOUSEMOVE: {
 
-			Point pt = Point::FromLong(lParam);
+			GUI::Point pt = PointFromLong(lParam);
 			TCHITTESTINFO thti;
 			thti.pt.x = pt.x;
 			thti.pt.y = pt.y;
@@ -823,9 +837,9 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 	case WM_PAINT: {
 			if (st_bDragBegin == TRUE && st_iDraggingTab != -1) {
 
-				Point ptCursor;
+				GUI::Point ptCursor;
 				::GetCursorPos(reinterpret_cast<POINT*>(&ptCursor));
-				Point ptClient = ptCursor;
+				GUI::Point ptClient = ptCursor;
 				::ScreenToClient(hWnd, reinterpret_cast<POINT*>(&ptClient));
 				TCHITTESTINFO thti;
 				thti.pt.x = ptClient.x;
@@ -839,40 +853,42 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 				        TabCtrl_GetItemRect(hWnd, tab, &tabrc)) {
 
 					HDC hDC = ::GetDC(hWnd);
-					Surface *surfaceWindow = Surface::Allocate();
-					if (surfaceWindow) {
-						surfaceWindow->Init(hDC, hWnd);
+					if (hDC) {
 
 						int xLeft = tabrc.left + 8;
 						int yLeft = tabrc.top + (tabrc.bottom - tabrc.top) / 2;
-						Point ptsLeftArrow[] = {
-							Point(xLeft, yLeft - 2),
-							Point(xLeft - 2, yLeft - 2),
-							Point(xLeft - 2, yLeft - 5),
-							Point(xLeft - 7, yLeft),
-							Point(xLeft - 2, yLeft + 5),
-							Point(xLeft - 2, yLeft + 2),
-							Point(xLeft, yLeft + 2)
+						POINT ptsLeftArrow[] = {
+							{xLeft, yLeft - 2},
+							{xLeft - 2, yLeft - 2},
+							{xLeft - 2, yLeft - 5},
+							{xLeft - 7, yLeft},
+							{xLeft - 2, yLeft + 5},
+							{xLeft - 2, yLeft + 2},
+							{xLeft, yLeft + 2}
 						};
 
 						int xRight = tabrc.right - 10;
 						int yRight = tabrc.top + (tabrc.bottom - tabrc.top) / 2;
-						Point ptsRightArrow[] = {
-							Point(xRight, yRight - 2),
-							Point(xRight + 2, yRight - 2),
-							Point(xRight + 2, yRight - 5),
-							Point(xRight + 7, yRight),
-							Point(xRight + 2, yRight + 5),
-							Point(xRight + 2, yRight + 2),
-							Point(xRight, yRight + 2)
+						POINT ptsRightArrow[] = {
+							{xRight, yRight - 2},
+							{xRight + 2, yRight - 2},
+							{xRight + 2, yRight - 5},
+							{xRight + 7, yRight},
+							{xRight + 2, yRight + 5},
+							{xRight + 2, yRight + 2},
+							{xRight, yRight + 2}
 						};
 
-						surfaceWindow->Polygon(tab < st_iDraggingTab ? ptsLeftArrow : ptsRightArrow,
-						        7,
-						        ColourAllocated(RGB(255, 0, 0)),
-						        ColourAllocated(RGB(255, 0, 0)));
-						surfaceWindow->Release();
-						delete surfaceWindow;
+						HPEN pen = ::CreatePen(0,1,RGB(255, 0, 0));
+						HPEN penOld = static_cast<HPEN>(::SelectObject(hDC, pen));
+						COLORREF colourNearest = ::GetNearestColor(hDC, RGB(255, 0, 0));
+						HBRUSH brush = ::CreateSolidBrush(colourNearest);
+						HBRUSH brushOld = static_cast<HBRUSH>(::SelectObject(hDC, brush));
+						::Polygon(hDC, tab < st_iDraggingTab ? ptsLeftArrow : ptsRightArrow, 7);
+						::SelectObject(hDC, brushOld);
+						::DeleteObject(brush);
+						::SelectObject(hDC, penOld);
+						::DeleteObject(pen);
 					}
 					::ReleaseDC(hWnd, hDC);
 				}
@@ -892,69 +908,57 @@ void SciTEWin::Creation() {
 	wContent = ::CreateWindowEx(
 	               WS_EX_CLIENTEDGE,
 	               classNameInternal,
-	               "Source",
+	               TEXT("Source"),
 	               WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
 	               0, 0,
 	               100, 100,
 	               MainHWND(),
 	               reinterpret_cast<HMENU>(2000),
 	               hInstance,
-	               reinterpret_cast<LPSTR>(this));
+	               reinterpret_cast<LPSTR>(&contents));
 	wContent.Show();
 
-	wEditor = ::CreateWindowEx(
+	wEditor.SetID(::CreateWindowEx(
 	              0,
-	              "Scintilla",
-	              "Source",
+	              TEXT("Scintilla"),
+	              TEXT("Source"),
 	              WS_CHILD | WS_VSCROLL | WS_HSCROLL | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
 	              0, 0,
 	              100, 100,
 	              reinterpret_cast<HWND>(wContent.GetID()),
 	              reinterpret_cast<HMENU>(IDM_SRCWIN),
 	              hInstance,
-	              0);
-	if (!wEditor.Created())
-		exit(FALSE);
-	fnEditor = reinterpret_cast<SciFnDirect>(::SendMessage(
-	               reinterpret_cast<HWND>(wEditor.GetID()), SCI_GETDIRECTFUNCTION, 0, 0));
-	ptrEditor = ::SendMessage(reinterpret_cast<HWND>(wEditor.GetID()),
-	                          SCI_GETDIRECTPOINTER, 0, 0);
-	if (!fnEditor || !ptrEditor)
+	              0));
+	if (!wEditor.CanCall())
 		exit(FALSE);
 	wEditor.Show();
-	SendEditor(SCI_USEPOPUP, 0);
+	wEditor.Call(SCI_USEPOPUP, 0);
 	WindowSetFocus(wEditor);
 
-	wOutput = ::CreateWindowEx(
+	wOutput.SetID(::CreateWindowEx(
 	              0,
-	              "Scintilla",
-	              "Run",
+	              TEXT("Scintilla"),
+	              TEXT("Run"),
 	              WS_CHILD | WS_VSCROLL | WS_HSCROLL | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
 	              0, 0,
 	              100, 100,
 	              reinterpret_cast<HWND>(wContent.GetID()),
 	              reinterpret_cast<HMENU>(IDM_RUNWIN),
 	              hInstance,
-	              0);
-	if (!wOutput.Created())
-		exit(FALSE);
-	fnOutput = reinterpret_cast<SciFnDirect>(::SendMessage(
-	               reinterpret_cast<HWND>(wOutput.GetID()), SCI_GETDIRECTFUNCTION, 0, 0));
-	ptrOutput = ::SendMessage(reinterpret_cast<HWND>(wOutput.GetID()),
-	                          SCI_GETDIRECTPOINTER, 0, 0);
-	if (!fnOutput || !ptrOutput)
+	              0));
+	if (!wOutput.CanCall())
 		exit(FALSE);
 	wOutput.Show();
 	// No selection margin on output window
-	SendOutput(SCI_SETMARGINWIDTHN, 1, 0);
-	//SendOutput(SCI_SETCARETPERIOD, 0);
-	SendOutput(SCI_USEPOPUP, 0);
+	wOutput.Call(SCI_SETMARGINWIDTHN, 1, 0);
+	//wOutput.Call(SCI_SETCARETPERIOD, 0);
+	wOutput.Call(SCI_USEPOPUP, 0);
 	::DragAcceptFiles(MainHWND(), true);
 
 	HWND hwndToolBar = ::CreateWindowEx(
 	               0,
 	               TOOLBARCLASSNAME,
-	               "",
+	               TEXT(""),
 	               WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS |
 	               TBSTYLE_FLAT | TBSTYLE_TOOLTIPS | CCS_NORESIZE,
 	               0, 0,
@@ -1002,15 +1006,15 @@ void SciTEWin::Creation() {
 	stDefaultTabProc = wndClass.lpfnWndProc;
 	wndClass.lpfnWndProc = TabWndProc;
 	wndClass.style = wndClass.style | CS_DBLCLKS;
-	wndClass.lpszClassName = "SciTeTabCtrl";
+	wndClass.lpszClassName = TEXT("SciTeTabCtrl");
 	wndClass.hInstance = hInstance;
 	if (RegisterClass(&wndClass) == 0)
 		exit(FALSE);
 
 	wTabBar = ::CreateWindowEx(
 	              0,
-	              "SciTeTabCtrl",
-	              "Tab",
+	              TEXT("SciTeTabCtrl"),
+	              TEXT("Tab"),
 	              WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS |
 	              TCS_FOCUSNEVER | TCS_TOOLTIPS,
 	              0, 0,
@@ -1034,10 +1038,49 @@ void SciTEWin::Creation() {
 
 	wTabBar.Show();
 
+	::CreateWindowEx(
+	               0,
+	               classNameInternal,
+	               TEXT("searchStrip"),
+	               WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
+	               0, 0,
+	               100, 100,
+	               MainHWND(),
+	               reinterpret_cast<HMENU>(2001),
+	               hInstance,
+	               reinterpret_cast<LPSTR>(&searchStrip));
+	//searchStrip.Show();
+
+	::CreateWindowEx(
+	               0,
+	               classNameInternal,
+	               TEXT("FindStrip"),
+	               WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
+	               0, 0,
+	               100, 100,
+	               MainHWND(),
+	               reinterpret_cast<HMENU>(2002),
+	               hInstance,
+	               reinterpret_cast<LPSTR>(&findStrip));
+	//findStrip.Show();
+
+	::CreateWindowEx(
+	               0,
+	               classNameInternal,
+	               TEXT("ReplaceStrip"),
+	               WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
+	               0, 0,
+	               100, 100,
+	               MainHWND(),
+	               reinterpret_cast<HMENU>(2003),
+	               hInstance,
+	               reinterpret_cast<LPSTR>(&replaceStrip));
+	//replaceStrip.Show();
+
 	wStatusBar = ::CreateWindowEx(
 	                 0,
 	                 STATUSCLASSNAME,
-	                 "",
+	                 TEXT(""),
 	                 WS_CHILD | WS_CLIPSIBLINGS,
 	                 0, 0,
 	                 100, heightStatus,
@@ -1053,6 +1096,14 @@ void SciTEWin::Creation() {
 	              SB_SETPARTS, 1,
 	              reinterpret_cast<LPARAM>(widths));
 
+	bands.push_back(Band(true, heightTools, false, wToolBar));
+	bands.push_back(Band(true, heightTab, false, wTabBar));
+	bands.push_back(Band(true, 100, true, wContent));
+	bands.push_back(Band(true, searchStrip.Height(), false, searchStrip));
+	bands.push_back(Band(true, findStrip.Height(), false, findStrip));
+	bands.push_back(Band(true, replaceStrip.Height(), false, replaceStrip));
+	bands.push_back(Band(true, heightStatus, false, wStatusBar));
+
 #ifndef NO_LUA
 		if (props.GetExpanded("ext.lua.startup.script").length() == 0)
 			DestroyMenuItem(menuOptions,IDM_OPENLUAEXTERNALFILE);
@@ -1060,4 +1111,3 @@ void SciTEWin::Creation() {
 		DestroyMenuItem(menuOptions,IDM_OPENLUAEXTERNALFILE);
 #endif
 }
-
