@@ -16,6 +16,7 @@
 #include <string>
 #include <vector>
 
+#undef _WIN32_WINNT
 #define _WIN32_WINNT  0x0500
 #include <windows.h>
 #include <commctrl.h>
@@ -24,13 +25,12 @@
 
 #include "Platform.h"
 
+#include "ILexer.h"
 #include "Scintilla.h"
+
 #ifdef SCI_LEXER
 #include "SciLexer.h"
-#include "PropSet.h"
-#include "PropSetSimple.h"
-#include "Accessor.h"
-#include "KeyWords.h"
+#include "LexerModule.h"
 #endif
 #include "SplitVector.h"
 #include "Partitioning.h"
@@ -75,11 +75,9 @@
 #endif
 
 #include <commctrl.h>
-#ifndef __BORLANDC__
 #ifndef __DMC__
 #include <zmouse.h>
 #endif
-#endif
 #include <ole2.h>
 
 #ifndef MK_ALT
@@ -93,11 +91,7 @@ extern bool IsNT();
 extern void Platform_Initialise(void *hInstance);
 extern void Platform_Finalise();
 
-/** TOTAL_CONTROL ifdef surrounds code that will only work when ScintillaWin
- * is derived from ScintillaBase (all features) rather than directly from Editor
- * (lightweight editor).
- */
-#define TOTAL_CONTROL
+typedef BOOL (WINAPI *TrackMouseEventSig)(LPTRACKMOUSEEVENT);
 
 // GCC has trouble with the standard COM ABI so do it the old C way with explicit vtables.
 
@@ -110,11 +104,13 @@ using namespace Scintilla;
 
 class ScintillaWin; 	// Forward declaration for COM interface subobjects
 
+typedef void VFunction(void);
+
 /**
  */
 class FormatEnumerator {
 public:
-	void **vtbl;
+	VFunction **vtbl;
 	int ref;
 	int pos;
 	CLIPFORMAT formats[2];
@@ -126,7 +122,7 @@ public:
  */
 class DropSource {
 public:
-	void **vtbl;
+	VFunction **vtbl;
 	ScintillaWin *sci;
 	DropSource();
 };
@@ -135,7 +131,7 @@ public:
  */
 class DataObject {
 public:
-	void **vtbl;
+	VFunction **vtbl;
 	ScintillaWin *sci;
 	DataObject();
 };
@@ -144,7 +140,7 @@ public:
  */
 class DropTarget {
 public:
-	void **vtbl;
+	VFunction **vtbl;
 	ScintillaWin *sci;
 	DropTarget();
 };
@@ -157,6 +153,8 @@ class ScintillaWin :
 	bool lastKeyDownConsumed;
 
 	bool capturedMouse;
+	bool trackedMouseLeave;
+	TrackMouseEventSig TrackMouseEventFn;
 
 	unsigned int linesPerScroll;	///< Intellimouse support
 	int wheelDelta; ///< Wheel delta from roll
@@ -197,12 +195,14 @@ class ScintillaWin :
 	virtual void StartDrag();
 	sptr_t WndPaint(uptr_t wParam);
 	sptr_t HandleComposition(uptr_t wParam, sptr_t lParam);
+	UINT CodePageOfDocument();
 	virtual bool ValidCodePage(int codePage) const;
 	virtual sptr_t DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 	virtual bool SetIdle(bool on);
 	virtual void SetTicking(bool on);
 	virtual void SetMouseCapture(bool on);
 	virtual bool HaveMouseCapture();
+	virtual void SetTrackMouseLeaveEvent(bool on);
 	virtual bool PaintContains(PRectangle rc);
 	virtual void ScrollText(int linesToMove);
 	virtual void UpdateSystemCaret();
@@ -214,6 +214,8 @@ class ScintillaWin :
 	virtual int GetCtrlID();
 	virtual void NotifyParent(SCNotification scn);
 	virtual void NotifyDoubleClick(Point pt, bool shift, bool ctrl, bool alt);
+	virtual CaseFolder *CaseFolderForEncoding();
+	virtual std::string CaseMapString(const std::string &s, int caseMapping);
 	virtual void Copy();
 	virtual void CopyAllowLine();
 	virtual bool CanPaste();
@@ -264,12 +266,6 @@ public:
 	/// Implement important part of IDataObject
 	STDMETHODIMP GetData(FORMATETC *pFEIn, STGMEDIUM *pSTM);
 
-	// External Lexers
-#ifdef SCI_LEXER
-	void SetLexerLanguage(const char *languageName);
-	void SetLexer(uptr_t wParam);
-#endif
-
 	static bool Register(HINSTANCE hInstance_);
 	static bool Unregister();
 
@@ -298,6 +294,9 @@ ScintillaWin::ScintillaWin(HWND hwnd) {
 	lastKeyDownConsumed = false;
 
 	capturedMouse = false;
+	trackedMouseLeave = false;
+	TrackMouseEventFn = 0;
+
 	linesPerScroll = 0;
 	wheelDelta = 0;   // Wheel delta from roll
 
@@ -328,6 +327,10 @@ ScintillaWin::ScintillaWin(HWND hwnd) {
 
 	keysAlwaysUnicode = false;
 
+	caret.period = ::GetCaretBlinkTime();
+	if (caret.period < 0)
+		caret.period = 0;
+
 	Initialise();
 }
 
@@ -338,6 +341,18 @@ void ScintillaWin::Initialise() {
 	// no effect.  If the app hasnt, we really shouldnt ask them to call
 	// it just so this internal feature works.
 	hrOle = ::OleInitialize(NULL);
+
+	// Find TrackMouseEvent which is available on Windows > 95
+	HMODULE user32 = ::GetModuleHandle(TEXT("user32.dll"));
+	TrackMouseEventFn = (TrackMouseEventSig)::GetProcAddress(user32, "TrackMouseEvent");
+	if (TrackMouseEventFn == NULL) {
+		// Windows 95 has an emulation in comctl32.dll:_TrackMouseEvent
+		HMODULE commctrl32 = ::LoadLibrary(TEXT("comctl32.dll"));
+		if (commctrl32 != NULL) {
+			TrackMouseEventFn = (TrackMouseEventSig)
+				::GetProcAddress(commctrl32, "_TrackMouseEvent");
+		}
+	}
 }
 
 void ScintillaWin::Finalise() {
@@ -483,7 +498,7 @@ LRESULT ScintillaWin::WndPaint(uptr_t wParam) {
 		hRgnUpdate = 0;
 	}
 
-	if(!IsOcxCtrl)
+	if (!IsOcxCtrl)
 		::EndPaint(MainHWND(), pps);
 	if (paintState == paintAbandoned) {
 		// Painting area was insufficient to cover new styling or brace highlight positions
@@ -571,6 +586,10 @@ static unsigned int SciMessageFromEM(unsigned int iMessage) {
 }
 
 static UINT CodePageFromCharSet(DWORD characterSet, UINT documentCodePage) {
+	if (documentCodePage == SC_CP_UTF8) {
+		// The system calls here are a little slow so avoid if known case.
+		return SC_CP_UTF8;
+	}
 	CHARSETINFO ci = { 0, 0, { { 0, 0, 0, 0 }, { 0, 0 } } };
 	BOOL bci = ::TranslateCharsetInfo((DWORD*)characterSet,
 		&ci, TCI_SRCCHARSET);
@@ -588,6 +607,10 @@ static UINT CodePageFromCharSet(DWORD characterSet, UINT documentCodePage) {
 	return cp;
 }
 
+UINT ScintillaWin::CodePageOfDocument() {
+	return CodePageFromCharSet(vs.styles[STYLE_DEFAULT].characterSet, pdoc->dbcsCodePage);
+}
+
 sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) {
 	try {
 		//Platform::DebugPrintf("S M:%x WP:%x L:%x\n", iMessage, wParam, lParam);
@@ -602,9 +625,7 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			break;
 
 		case WM_COMMAND:
-#ifdef TOTAL_CONTROL
 			Command(LoWord(wParam));
-#endif
 			break;
 
 		case WM_PAINT:
@@ -685,7 +706,7 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 		case SC_WIN_IDLE:
 			// wParam=dwTickCountInitial, or 0 to initialize.  lParam=bSkipUserInputTest
 			if (idler.state) {
-				if (lParam || (WAIT_TIMEOUT==MsgWaitForMultipleObjects(0,0,0,0, QS_INPUT|QS_HOTKEY))) {
+				if (lParam || (WAIT_TIMEOUT == MsgWaitForMultipleObjects(0, 0, 0, 0, QS_INPUT|QS_HOTKEY))) {
 					if (Idle()) {
 						// User input was given priority above, but all events do get a turn.  Other
 						// messages, notifications, etc. will get interleaved with the idle messages.
@@ -722,18 +743,24 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			//	Platform::IsKeyDown(VK_SHIFT),
 			//	Platform::IsKeyDown(VK_CONTROL),
 			//	Platform::IsKeyDown(VK_MENU));
+			::SetFocus(MainHWND());
 			ButtonDown(Point::FromLong(lParam), ::GetMessageTime(),
 				(wParam & MK_SHIFT) != 0,
 				(wParam & MK_CONTROL) != 0,
 				Platform::IsKeyDown(VK_MENU));
-			::SetFocus(MainHWND());
 			}
 			break;
 
 		case WM_MOUSEMOVE:
+			SetTrackMouseLeaveEvent(true);
 			ButtonMove(Point::FromLong(lParam));
 			break;
 
+		case WM_MOUSELEAVE:
+			SetTrackMouseLeaveEvent(false);
+			MouseLeave();
+			return ::DefWindowProc(MainHWND(), iMessage, wParam, lParam);
+
 		case WM_LBUTTONUP:
 			ButtonUp(Point::FromLong(lParam),
 				::GetMessageTime(),
@@ -741,8 +768,11 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			break;
 
 		case WM_RBUTTONDOWN:
-			if (!PointInSelection(Point::FromLong(lParam)))
+			::SetFocus(MainHWND());
+			if (!PointInSelection(Point::FromLong(lParam))) {
+				CancelModes();
 				SetEmptySelection(PositionFromLocation(Point::FromLong(lParam)));
+			}
 			break;
 
 		case WM_SETCURSOR:
@@ -755,7 +785,7 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 					::GetCursorPos(&pt);
 					::ScreenToClient(MainHWND(), &pt);
 					if (PointInSelMargin(Point(pt.x, pt.y))) {
-						DisplayCursor(Window::cursorReverseArrow);
+						DisplayCursor(GetMarginCursor(Point(pt.x, pt.y)));
 					} else if (PointInSelection(Point(pt.x, pt.y)) && !SelectionEmpty()) {
 						DisplayCursor(Window::cursorArrow);
 					} else if (PointIsHotspot(Point(pt.x, pt.y))) {
@@ -780,11 +810,11 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 						UTF8FromUTF16(wcs, 1, utfval, len);
 						AddCharUTF(utfval, len);
 					} else {
-						UINT cpDest = CodePageFromCharSet(
-							vs.styles[STYLE_DEFAULT].characterSet, pdoc->dbcsCodePage);
+						UINT cpDest = CodePageOfDocument();
 						char inBufferCP[20];
 						int size = ::WideCharToMultiByte(cpDest,
 							0, wcs, 1, inBufferCP, sizeof(inBufferCP) - 1, 0, 0);
+						inBufferCP[size] = '\0';
 						AddCharUTF(inBufferCP, size);
 					}
 				} else {
@@ -852,7 +882,7 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 				HWND wThis = MainHWND();
 				HWND wCT = reinterpret_cast<HWND>(ct.wCallTip.GetID());
 				if (!wParam ||
-					!(::IsChild(wThis,wOther) || (wOther == wCT))) {
+					!(::IsChild(wThis, wOther) || (wOther == wCT))) {
 					SetFocusState(false);
 					DestroySystemCaret();
 				}
@@ -901,7 +931,6 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 			}
 
 		case WM_CONTEXTMENU:
-#ifdef TOTAL_CONTROL
 			if (displayPopupMenu) {
 				Point pt = Point::FromLong(lParam);
 				if ((pt.x == -1) && (pt.y == -1)) {
@@ -914,7 +943,6 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 				ContextMenu(pt);
 				return 0;
 			}
-#endif
 			return ::DefWindowProc(MainHWND(), iMessage, wParam, lParam);
 
 		case WM_INPUTLANGCHANGE:
@@ -1028,14 +1056,14 @@ sptr_t ScintillaWin::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam
 
 #ifdef SCI_LEXER
 		case SCI_LOADLEXERLIBRARY:
-			LexerManager::GetInstance()->Load(reinterpret_cast<const char*>(lParam));
+			LexerManager::GetInstance()->Load(reinterpret_cast<const char *>(lParam));
 			break;
 #endif
 
 		default:
 			return ScintillaBase::WndProc(iMessage, wParam, lParam);
 		}
-	} catch (std::bad_alloc&) {
+	} catch (std::bad_alloc &) {
 		errorStatus = SC_STATUS_BADALLOC;
 	} catch (...) {
 		errorStatus = SC_STATUS_FAILURE;
@@ -1101,6 +1129,17 @@ bool ScintillaWin::HaveMouseCapture() {
 	//return capturedMouse && (::GetCapture() == MainHWND());
 }
 
+void ScintillaWin::SetTrackMouseLeaveEvent(bool on) {
+	if (on && TrackMouseEventFn && !trackedMouseLeave) {
+		TRACKMOUSEEVENT tme;
+		tme.cbSize = sizeof(tme);
+		tme.dwFlags = TME_LEAVE;
+		tme.hwndTrack = MainHWND();
+		TrackMouseEventFn(&tme);
+	}
+	trackedMouseLeave = on;
+}
+
 bool ScintillaWin::PaintContains(PRectangle rc) {
 	bool contains = true;
 	if ((paintState == painting) && (!rc.Empty())) {
@@ -1154,7 +1193,7 @@ bool ScintillaWin::GetScrollInfo(int nBar, LPSCROLLINFO lpsi) {
 // Change the scroll position but avoid repaint if changing to same value
 void ScintillaWin::ChangeScrollPos(int barType, int pos) {
 	SCROLLINFO sci = {
-		sizeof(sci),0,0,0,0,0,0
+		sizeof(sci), 0, 0, 0, 0, 0, 0
 	};
 	sci.fMask = SIF_POS;
 	GetScrollInfo(barType, &sci);
@@ -1176,7 +1215,7 @@ void ScintillaWin::SetHorizontalScrollPos() {
 bool ScintillaWin::ModifyScrollBars(int nMax, int nPage) {
 	bool modified = false;
 	SCROLLINFO sci = {
-		sizeof(sci),0,0,0,0,0,0
+		sizeof(sci), 0, 0, 0, 0, 0, 0
 	};
 	sci.fMask = SIF_PAGE | SIF_RANGE;
 	GetScrollInfo(SB_VERT, &sci);
@@ -1260,6 +1299,211 @@ void ScintillaWin::NotifyDoubleClick(Point pt, bool shift, bool ctrl, bool alt)
 			  MAKELPARAM(pt.x, pt.y));
 }
 
+class CaseFolderUTF8 : public CaseFolderTable {
+	// Allocate the expandable storage here so that it does not need to be reallocated
+	// for each call to Fold.
+	std::vector<wchar_t> utf16Mixed;
+	std::vector<wchar_t> utf16Folded;
+public:
+	CaseFolderUTF8() {
+		StandardASCII();
+	}
+	virtual size_t Fold(char *folded, size_t sizeFolded, const char *mixed, size_t lenMixed) {
+		if ((lenMixed == 1) && (sizeFolded > 0)) {
+			folded[0] = mapping[static_cast<unsigned char>(mixed[0])];
+			return 1;
+		} else {
+			if (lenMixed > utf16Mixed.size()) {
+				utf16Mixed.resize(lenMixed + 8);
+			}
+			size_t nUtf16Mixed = ::MultiByteToWideChar(65001, 0, mixed, lenMixed,
+				&utf16Mixed[0], utf16Mixed.size());
+
+			if (nUtf16Mixed == 0) {
+				// Failed to convert -> bad UTF-8
+				folded[0] = '\0';
+				return 1;
+			}
+
+			if (nUtf16Mixed * 4 > utf16Folded.size()) {	// Maximum folding expansion factor of 4
+				utf16Folded.resize(nUtf16Mixed * 4 + 8);
+			}
+			int lenFlat = ::LCMapStringW(LOCALE_SYSTEM_DEFAULT,
+				LCMAP_LINGUISTIC_CASING | LCMAP_LOWERCASE,
+				&utf16Mixed[0], nUtf16Mixed, &utf16Folded[0], utf16Folded.size());
+
+			size_t lenOut = UTF8Length(&utf16Folded[0], lenFlat);
+			if (lenOut < sizeFolded) {
+				UTF8FromUTF16(&utf16Folded[0], lenFlat, folded, lenOut);
+				return lenOut;
+			} else {
+				return 0;
+			}
+		}
+	}
+};
+
+class CaseFolderDBCS : public CaseFolderTable {
+	// Allocate the expandable storage here so that it does not need to be reallocated
+	// for each call to Fold.
+	std::vector<wchar_t> utf16Mixed;
+	std::vector<wchar_t> utf16Folded;
+	UINT cp;
+public:
+	CaseFolderDBCS(UINT cp_) : cp(cp_) {
+		StandardASCII();
+	}
+	virtual size_t Fold(char *folded, size_t sizeFolded, const char *mixed, size_t lenMixed) {
+		if ((lenMixed == 1) && (sizeFolded > 0)) {
+			folded[0] = mapping[static_cast<unsigned char>(mixed[0])];
+			return 1;
+		} else {
+			if (lenMixed > utf16Mixed.size()) {
+				utf16Mixed.resize(lenMixed + 8);
+			}
+			size_t nUtf16Mixed = ::MultiByteToWideChar(cp, 0, mixed, lenMixed,
+				&utf16Mixed[0], utf16Mixed.size());
+
+			if (nUtf16Mixed == 0) {
+				// Failed to convert -> bad input
+				folded[0] = '\0';
+				return 1;
+			}
+
+			if (nUtf16Mixed * 4 > utf16Folded.size()) {	// Maximum folding expansion factor of 4
+				utf16Folded.resize(nUtf16Mixed * 4 + 8);
+			}
+			int lenFlat = ::LCMapStringW(LOCALE_SYSTEM_DEFAULT,
+				LCMAP_LINGUISTIC_CASING | LCMAP_LOWERCASE,
+				&utf16Mixed[0], nUtf16Mixed, &utf16Folded[0], utf16Folded.size());
+
+			size_t lenOut = ::WideCharToMultiByte(cp, 0,
+				&utf16Folded[0], lenFlat,
+				NULL, 0, NULL, 0);
+
+			if (lenOut < sizeFolded) {
+				::WideCharToMultiByte(cp, 0,
+					&utf16Folded[0], lenFlat,
+					folded, lenOut, NULL, 0);
+				return lenOut;
+			} else {
+				return 0;
+			}
+		}
+	}
+};
+
+CaseFolder *ScintillaWin::CaseFolderForEncoding() {
+	UINT cpDest = CodePageOfDocument();
+	if (cpDest == SC_CP_UTF8) {
+		return new CaseFolderUTF8();
+	} else {
+		if (pdoc->dbcsCodePage == 0) {
+			CaseFolderTable *pcf = new CaseFolderTable();
+			pcf->StandardASCII();
+			// Only for single byte encodings
+			UINT cpDoc = CodePageOfDocument();
+			for (int i=0x80; i<0x100; i++) {
+				char sCharacter[2] = "A";
+				sCharacter[0] = static_cast<char>(i);
+				wchar_t wCharacter[20];
+				unsigned int lengthUTF16 = ::MultiByteToWideChar(cpDoc, 0, sCharacter, 1,
+					wCharacter, sizeof(wCharacter)/sizeof(wCharacter[0]));
+				if (lengthUTF16 == 1) {
+					wchar_t wLower[20];
+					int charsConverted = ::LCMapStringW(LOCALE_SYSTEM_DEFAULT,
+						LCMAP_LINGUISTIC_CASING | LCMAP_LOWERCASE,
+						wCharacter, lengthUTF16, wLower, sizeof(wLower)/sizeof(wLower[0]));
+					char sCharacterLowered[20];
+					unsigned int lengthConverted = ::WideCharToMultiByte(cpDoc, 0,
+						wLower, charsConverted,
+						sCharacterLowered, sizeof(sCharacterLowered), NULL, 0);
+					if ((lengthConverted == 1) && (sCharacter[0] != sCharacterLowered[0])) {
+						pcf->SetTranslation(sCharacter[0], sCharacterLowered[0]);
+					}
+				}
+			}
+			return pcf;
+		} else {
+			return new CaseFolderDBCS(cpDest);
+		}
+	}
+}
+
+std::string ScintillaWin::CaseMapString(const std::string &s, int caseMapping) {
+	if (s.size() == 0)
+		return std::string();
+
+	if (caseMapping == cmSame)
+		return s;
+
+	UINT cpDoc = CodePageOfDocument();
+
+	unsigned int lengthUTF16 = ::MultiByteToWideChar(cpDoc, 0, s.c_str(), s.size(), NULL, 0);
+	if (lengthUTF16 == 0)	// Failed to convert
+		return s;
+
+	DWORD mapFlags = LCMAP_LINGUISTIC_CASING |
+		((caseMapping == cmUpper) ? LCMAP_UPPERCASE : LCMAP_LOWERCASE);
+
+	// Many conversions performed by search function are short so optimize this case.
+	enum { shortSize=20 };
+
+	if (s.size() > shortSize) {
+		// Use dynamic allocations for long strings
+
+		// Change text to UTF-16
+		std::vector<wchar_t> vwcText(lengthUTF16);
+		::MultiByteToWideChar(cpDoc, 0, s.c_str(), s.size(), &vwcText[0], lengthUTF16);
+
+		// Change case
+		int charsConverted = ::LCMapStringW(LOCALE_SYSTEM_DEFAULT, mapFlags,
+			&vwcText[0], lengthUTF16, NULL, 0);
+		std::vector<wchar_t> vwcConverted(charsConverted);
+		::LCMapStringW(LOCALE_SYSTEM_DEFAULT, mapFlags,
+			&vwcText[0], lengthUTF16, &vwcConverted[0], charsConverted);
+
+		// Change back to document encoding
+		unsigned int lengthConverted = ::WideCharToMultiByte(cpDoc, 0,
+			&vwcConverted[0], vwcConverted.size(),
+			NULL, 0, NULL, 0);
+		std::vector<char> vcConverted(lengthConverted);
+		::WideCharToMultiByte(cpDoc, 0,
+			&vwcConverted[0], vwcConverted.size(),
+			&vcConverted[0], vcConverted.size(), NULL, 0);
+
+		return std::string(&vcConverted[0], vcConverted.size());
+
+	} else {
+		// Use static allocations for short strings as much faster
+		// A factor of 15 for single character strings
+
+		// Change text to UTF-16
+		wchar_t vwcText[shortSize];
+		::MultiByteToWideChar(cpDoc, 0, s.c_str(), s.size(), vwcText, lengthUTF16);
+
+		// Change case
+		int charsConverted = ::LCMapStringW(LOCALE_SYSTEM_DEFAULT, mapFlags,
+			vwcText, lengthUTF16, NULL, 0);
+		// Full mapping may produce up to 3 characters per input character
+		wchar_t vwcConverted[shortSize*3];
+		::LCMapStringW(LOCALE_SYSTEM_DEFAULT, mapFlags, vwcText, lengthUTF16,
+			vwcConverted, charsConverted);
+
+		// Change back to document encoding
+		unsigned int lengthConverted = ::WideCharToMultiByte(cpDoc, 0,
+			vwcConverted, charsConverted,
+			NULL, 0, NULL, 0);
+		// Each UTF-16 code unit may need up to 3 bytes in UTF-8
+		char vcConverted[shortSize * 3 * 3];
+		::WideCharToMultiByte(cpDoc, 0,
+			vwcConverted, charsConverted,
+			vcConverted, lengthConverted, NULL, 0);
+
+		return std::string(vcConverted, lengthConverted);
+	}
+}
+
 void ScintillaWin::Copy() {
 	//Platform::DebugPrintf("Copy\n");
 	if (!sel.Empty()) {
@@ -1316,7 +1560,7 @@ public:
 	void SetClip(UINT uFormat) {
 		::SetClipboardData(uFormat, Unlock());
 	}
-	operator bool() {
+	operator bool() const {
 		return ptr != 0;
 	}
 	SIZE_T Size() {
@@ -1347,10 +1591,7 @@ void ScintillaWin::InsertPasteText(const char *text, int len, SelectionPosition
 				SetEmptySelection(sel.MainCaret() + len);
 			}
 		} else {
-			selStart = SelectionPosition(InsertSpace(selStart.Position(), selStart.VirtualSpace()));
-			if (pdoc->InsertString(selStart.Position(), text, len)) {
-				SetEmptySelection(selStart.Position() + len);
-			}
+			InsertPaste(selStart, text, len);
 		}
 		delete []convertedText;
 	}
@@ -1361,7 +1602,7 @@ void ScintillaWin::Paste() {
 		return;
 	UndoGroup ug(pdoc);
 	bool isLine = SelectionEmpty() && (::IsClipboardFormatAvailable(cfLineSelect) != 0);
-	ClearSelection();
+	ClearSelection(multiPasteMode == SC_MULTIPASTE_EACH);
 	SelectionPosition selStart = sel.IsRectangular() ?
 		sel.Rectangular().Start() :
 		sel.Range(sel.Main()).Start();
@@ -1383,8 +1624,7 @@ void ScintillaWin::Paste() {
 			} else {
 				// CF_UNICODETEXT available, but not in Unicode mode
 				// Convert from Unicode to current Scintilla code page
-				UINT cpDest = CodePageFromCharSet(
-					vs.styles[STYLE_DEFAULT].characterSet, pdoc->dbcsCodePage);
+				UINT cpDest = CodePageOfDocument();
 				len = ::WideCharToMultiByte(cpDest, 0, uptr, -1,
 				                            NULL, 0, NULL, NULL) - 1; // subtract 0 terminator
 				putf = new char[len + 1];
@@ -1441,7 +1681,6 @@ void ScintillaWin::Paste() {
 }
 
 void ScintillaWin::CreateCallTipWindow(PRectangle) {
-#ifdef TOTAL_CONTROL
 	if (!ct.wCallTip.Created()) {
 		ct.wCallTip = ::CreateWindow(callClassName, TEXT("ACallTip"),
 					     WS_POPUP, 100, 100, 150, 20,
@@ -1450,11 +1689,9 @@ void ScintillaWin::CreateCallTipWindow(PRectangle) {
 					     this);
 		ct.wDraw = ct.wCallTip;
 	}
-#endif
 }
 
 void ScintillaWin::AddToPopUp(const char *label, int cmd, bool enabled) {
-#ifdef TOTAL_CONTROL
 	HMENU hmenuPopup = reinterpret_cast<HMENU>(popup.GetID());
 	if (!label[0])
 		::AppendMenuA(hmenuPopup, MF_SEPARATOR, 0, "");
@@ -1462,42 +1699,12 @@ void ScintillaWin::AddToPopUp(const char *label, int cmd, bool enabled) {
 		::AppendMenuA(hmenuPopup, MF_STRING, cmd, label);
 	else
 		::AppendMenuA(hmenuPopup, MF_STRING | MF_DISABLED | MF_GRAYED, cmd, label);
-#endif
 }
 
 void ScintillaWin::ClaimSelection() {
 	// Windows does not have a primary selection
 }
 
-#ifdef SCI_LEXER
-
-/*
-
-  Initial Windows-Only implementation of the external lexer
-  system in ScintillaWin class. Intention is to create a LexerModule
-  subclass (?) to have lex and fold methods which will call out to their
-  relevant DLLs...
-
-*/
-
-void ScintillaWin::SetLexer(uptr_t wParam) {
-	lexLanguage = wParam;
-	lexCurrent = LexerModule::Find(lexLanguage);
-	if (!lexCurrent)
-		lexCurrent = LexerModule::Find(SCLEX_NULL);
-}
-
-void ScintillaWin::SetLexerLanguage(const char *languageName) {
-	lexLanguage = SCLEX_CONTAINER;
-	lexCurrent = LexerModule::Find(languageName);
-	if (!lexCurrent)
-		lexCurrent = LexerModule::Find(SCLEX_NULL);
-	if (lexCurrent)
-		lexLanguage = lexCurrent->GetLanguage();
-}
-
-#endif
-
 /// Implement IUnknown
 
 STDMETHODIMP_(ULONG)FormatEnumerator_AddRef(FormatEnumerator *fe);
@@ -1561,14 +1768,14 @@ STDMETHODIMP FormatEnumerator_Clone(FormatEnumerator *fe, IEnumFORMATETC **ppenu
 	                                       reinterpret_cast<void **>(ppenum));
 }
 
-static void *vtFormatEnumerator[] = {
-	(void *)(FormatEnumerator_QueryInterface),
-	(void *)(FormatEnumerator_AddRef),
-	(void *)(FormatEnumerator_Release),
-	(void *)(FormatEnumerator_Next),
-	(void *)(FormatEnumerator_Skip),
-	(void *)(FormatEnumerator_Reset),
-	(void *)(FormatEnumerator_Clone)
+static VFunction *vtFormatEnumerator[] = {
+	(VFunction *)(FormatEnumerator_QueryInterface),
+	(VFunction *)(FormatEnumerator_AddRef),
+	(VFunction *)(FormatEnumerator_Release),
+	(VFunction *)(FormatEnumerator_Next),
+	(VFunction *)(FormatEnumerator_Skip),
+	(VFunction *)(FormatEnumerator_Reset),
+	(VFunction *)(FormatEnumerator_Clone)
 };
 
 FormatEnumerator::FormatEnumerator(int pos_, CLIPFORMAT formats_[], int formatsLen_) {
@@ -1576,7 +1783,7 @@ FormatEnumerator::FormatEnumerator(int pos_, CLIPFORMAT formats_[], int formatsL
 	ref = 0;   // First QI adds first reference...
 	pos = pos_;
 	formatsLen = formatsLen_;
-	for (int i=0;i<formatsLen;i++)
+	for (int i=0; i<formatsLen; i++)
 		formats[i] = formats_[i];
 }
 
@@ -1604,12 +1811,12 @@ STDMETHODIMP DropSource_GiveFeedback(DropSource *, DWORD) {
 	return DRAGDROP_S_USEDEFAULTCURSORS;
 }
 
-static void *vtDropSource[] = {
-	(void *)(DropSource_QueryInterface),
-	(void *)(DropSource_AddRef),
-	(void *)(DropSource_Release),
-	(void *)(DropSource_QueryContinueDrag),
-	(void *)(DropSource_GiveFeedback)
+static VFunction *vtDropSource[] = {
+	(VFunction *)(DropSource_QueryInterface),
+	(VFunction *)(DropSource_AddRef),
+	(VFunction *)(DropSource_Release),
+	(VFunction *)(DropSource_QueryContinueDrag),
+	(VFunction *)(DropSource_GiveFeedback)
 };
 
 DropSource::DropSource() {
@@ -1699,7 +1906,7 @@ STDMETHODIMP DataObject_EnumFormatEtc(DataObject *pd, DWORD dwDirection, IEnumFO
 		}
 		return FormatEnumerator_QueryInterface(pfe, IID_IEnumFORMATETC,
 											   reinterpret_cast<void **>(ppEnum));
-	} catch (std::bad_alloc&) {
+	} catch (std::bad_alloc &) {
 		pd->sci->errorStatus = SC_STATUS_BADALLOC;
 		return E_OUTOFMEMORY;
 	} catch (...) {
@@ -1723,19 +1930,19 @@ STDMETHODIMP DataObject_EnumDAdvise(DataObject *, IEnumSTATDATA **) {
 	return E_FAIL;
 }
 
-static void *vtDataObject[] = {
-	(void *)(DataObject_QueryInterface),
-	(void *)(DataObject_AddRef),
-	(void *)(DataObject_Release),
-	(void *)(DataObject_GetData),
-	(void *)(DataObject_GetDataHere),
-	(void *)(DataObject_QueryGetData),
-	(void *)(DataObject_GetCanonicalFormatEtc),
-	(void *)(DataObject_SetData),
-	(void *)(DataObject_EnumFormatEtc),
-	(void *)(DataObject_DAdvise),
-	(void *)(DataObject_DUnadvise),
-	(void *)(DataObject_EnumDAdvise)
+static VFunction *vtDataObject[] = {
+	(VFunction *)(DataObject_QueryInterface),
+	(VFunction *)(DataObject_AddRef),
+	(VFunction *)(DataObject_Release),
+	(VFunction *)(DataObject_GetData),
+	(VFunction *)(DataObject_GetDataHere),
+	(VFunction *)(DataObject_QueryGetData),
+	(VFunction *)(DataObject_GetCanonicalFormatEtc),
+	(VFunction *)(DataObject_SetData),
+	(VFunction *)(DataObject_EnumFormatEtc),
+	(VFunction *)(DataObject_DAdvise),
+	(VFunction *)(DataObject_DUnadvise),
+	(VFunction *)(DataObject_EnumDAdvise)
 };
 
 DataObject::DataObject() {
@@ -1791,14 +1998,14 @@ STDMETHODIMP DropTarget_Drop(DropTarget *dt, LPDATAOBJECT pIDataSource, DWORD gr
 	return E_FAIL;
 }
 
-static void *vtDropTarget[] = {
-	(void *)(DropTarget_QueryInterface),
-	(void *)(DropTarget_AddRef),
-	(void *)(DropTarget_Release),
-	(void *)(DropTarget_DragEnter),
-	(void *)(DropTarget_DragOver),
-	(void *)(DropTarget_DragLeave),
-	(void *)(DropTarget_Drop)
+static VFunction *vtDropTarget[] = {
+	(VFunction *)(DropTarget_QueryInterface),
+	(VFunction *)(DropTarget_AddRef),
+	(VFunction *)(DropTarget_Release),
+	(VFunction *)(DropTarget_DragEnter),
+	(VFunction *)(DropTarget_DragOver),
+	(VFunction *)(DropTarget_DragLeave),
+	(VFunction *)(DropTarget_Drop)
 };
 
 DropTarget::DropTarget() {
@@ -1829,7 +2036,7 @@ void ScintillaWin::ImeStartComposition() {
 			// Since the style creation code has been made platform independent,
 			// The logfont for the IME is recreated here.
 			int styleHere = (pdoc->StyleAt(sel.MainCaret())) & 31;
-			LOGFONTA lf = {0,0,0,0,0,0,0,0,0,0,0,0,0, ""};
+			LOGFONTA lf = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, ""};
 			int sizeZoomed = vs.styles[styleHere].size + vs.zoomLevel;
 			if (sizeZoomed <= 2)	// Hangs if sizeZoomed <= 1
 				sizeZoomed = 2;
@@ -1865,7 +2072,7 @@ void ScintillaWin::AddCharBytes(char b0, char b1) {
 
 	int inputCodePage = InputCodePage();
 	if (inputCodePage && IsUnicodeMode()) {
-		char utfval[4]="\0\0\0";
+		char utfval[4] = "\0\0\0";
 		char ansiChars[3];
 		wchar_t wcs[2];
 		if (b0) {	// Two bytes from IME
@@ -1900,7 +2107,7 @@ void ScintillaWin::GetIntelliMouseParameters() {
 
 void ScintillaWin::CopyToClipboard(const SelectionText &selectedText) {
 	if (!::OpenClipboard(MainHWND()))
-		return ;
+		return;
 	::EmptyClipboard();
 
 	GlobalMemory uniText;
@@ -1917,10 +2124,11 @@ void ScintillaWin::CopyToClipboard(const SelectionText &selectedText) {
 		// Convert to Unicode using the current Scintilla code page
 		UINT cpSrc = CodePageFromCharSet(
 					selectedText.characterSet, selectedText.codePage);
-		uniText.Allocate(2 * selectedText.len);
+		int uLen = ::MultiByteToWideChar(cpSrc, 0, selectedText.s, selectedText.len, 0, 0);
+		uniText.Allocate(2 * uLen);
 		if (uniText) {
 			::MultiByteToWideChar(cpSrc, 0, selectedText.s, selectedText.len,
-				static_cast<wchar_t *>(uniText.ptr), selectedText.len);
+				static_cast<wchar_t *>(uniText.ptr), uLen);
 		}
 	}
 
@@ -2192,7 +2400,7 @@ STDMETHODIMP ScintillaWin::Drop(LPDATAOBJECT pIDataSource, DWORD grfKeyState,
 
 		SetDragPosition(SelectionPosition(invalidPosition));
 
-		STGMEDIUM medium={0,{0},0};
+		STGMEDIUM medium = {0, {0}, 0};
 
 		char *data = 0;
 		bool dataAllocated = false;
@@ -2214,8 +2422,7 @@ STDMETHODIMP ScintillaWin::Drop(LPDATAOBJECT pIDataSource, DWORD grfKeyState,
 				// Default Scintilla behavior in Unicode mode
 				// CF_UNICODETEXT available, but not in Unicode mode
 				// Convert from Unicode to current Scintilla code page
-				UINT cpDest = CodePageFromCharSet(
-					vs.styles[STYLE_DEFAULT].characterSet, pdoc->dbcsCodePage);
+				UINT cpDest = CodePageOfDocument();
 				int tlen = ::WideCharToMultiByte(cpDest, 0, udata, -1,
 					NULL, 0, NULL, NULL) - 1; // subtract 0 terminator
 				data = new char[tlen + 1];
@@ -2234,6 +2441,16 @@ STDMETHODIMP ScintillaWin::Drop(LPDATAOBJECT pIDataSource, DWORD grfKeyState,
 			}
 		}
 
+		if (data && convertPastes) {
+			// Convert line endings of the drop into our local line-endings mode
+			int len = strlen(data);
+			char *convertedText = Document::TransformLineEnds(&len, data, len, pdoc->eolMode);
+			if (dataAllocated)
+				delete []data;
+			data = convertedText;
+			dataAllocated = true;
+		}
+
 		if (!data) {
 			//Platform::DebugPrintf("Bad data format: 0x%x\n", hres);
 			return hr;
@@ -2244,7 +2461,7 @@ STDMETHODIMP ScintillaWin::Drop(LPDATAOBJECT pIDataSource, DWORD grfKeyState,
 
 		POINT rpt = {pt.x, pt.y};
 		::ScreenToClient(MainHWND(), &rpt);
-		SelectionPosition movePos = SPositionFromLocation(Point(rpt.x, rpt.y), false, false, UserVirtualSpace()); 
+		SelectionPosition movePos = SPositionFromLocation(Point(rpt.x, rpt.y), false, false, UserVirtualSpace());
 
 		DropAt(movePos, data, *pdwEffect == DROPEFFECT_MOVE, hrRectangular == S_OK);
 
@@ -2374,7 +2591,7 @@ bool ScintillaWin::Unregister() {
 bool ScintillaWin::HasCaretSizeChanged() {
 	if (
 		( (0 != vs.caretWidth) && (sysCaretWidth != vs.caretWidth) )
-		|| (0 != vs.lineHeight) && (sysCaretHeight != vs.lineHeight)
+		|| ((0 != vs.lineHeight) && (sysCaretHeight != vs.lineHeight))
 		) {
 		return true;
 	}
@@ -2475,7 +2692,7 @@ sptr_t PASCAL ScintillaWin::CTWndProc(
 				sciThis->CallTipClick();
 				return 0;
 			} else if (iMessage == WM_SETCURSOR) {
-				::SetCursor(::LoadCursor(NULL,IDC_ARROW));
+				::SetCursor(::LoadCursor(NULL, IDC_ARROW));
 				return 0;
 			} else if (iMessage == WM_NCHITTEST) {
 				return HTCAPTION;
@@ -2518,7 +2735,7 @@ sptr_t PASCAL ScintillaWin::SWndProc(
 				sci = new ScintillaWin(hWnd);
 				SetWindowPointer(hWnd, sci);
 				return sci->WndProc(iMessage, wParam, lParam);
-			} 
+			}
 		} catch (...) {
 		}
 		return ::DefWindowProc(hWnd, iMessage, wParam, lParam);
@@ -2539,7 +2756,7 @@ sptr_t PASCAL ScintillaWin::SWndProc(
 
 // This function is externally visible so it can be called from container when building statically.
 // Must be called once only.
-bool Scintilla_RegisterClasses(void *hInstance) {
+int Scintilla_RegisterClasses(void *hInstance) {
 	Platform_Initialise(hInstance);
 	bool result = ScintillaWin::Register(reinterpret_cast<HINSTANCE>(hInstance));
 #ifdef SCI_LEXER
@@ -2549,7 +2766,7 @@ bool Scintilla_RegisterClasses(void *hInstance) {
 }
 
 // This function is externally visible so it can be called from container when building statically.
-bool Scintilla_ReleaseResources() {
+int Scintilla_ReleaseResources() {
 	bool result = ScintillaWin::Unregister();
 	Platform_Finalise();
 	return result;
